/// <reference types="react" />
import { ImageProps, InputProps, Style, TextProps, UploadProps, ViewProps, StyleProperties, AsProp, Length, SxProp, ColorRGB } from "@compositive/primitives";
import { Color as PrimitiveColor } from "@compositive/primitives";
import { RefAttributes, ReactElement, ReactNode, SyntheticEvent, Ref, DependencyList } from "react";
import { NullablePartial, Merge, HKT } from "@compositive/commons-types";
import { AttributeDefinitions, AttributeValueDefinition, ChainEvaluator, Evaluator } from "@compositive/commons-spec";
import { Equals, Predicate } from "@compositive/commons-predicates";
import { Indexed, Sequence } from "@compositive/commons-sequences";
/**
 * A class representing parts of a surface, including prepended and appended
 * components, and container styles. This class provides methods to manipulate
 * these parts immutably.
 */
declare class SurfaceParts<T extends UnknownTheme> {
    #private;
    /**
     * Gets the list of prepended components.
     */
    get prependedComponents(): ReactElement[];
    /**
     * Prepends a component to the list of prepended components.
     *
     * @param component - The component to prepend.
     * @returns A new instance of SurfaceParts with the updated list of prepended
     *   components.
     */
    prependComponent(component: ReactElement): SurfaceParts<T>;
    /**
     * Gets the list of appended components.
     */
    get appendedComponents(): ReactElement[];
    /**
     * Appends a component to the list of appended components.
     *
     * @param component - The component to append.
     * @returns A new instance of SurfaceParts with the updated list of appended
     *   components.
     */
    appendComponent(component: ReactElement): SurfaceParts<T>;
    /**
     * Gets the container styles.
     */
    get containerStyles(): ThemedSxProp<T>;
    /**
     * Adds styles to the container.
     *
     * @param styles - The styles to add.
     * @returns A new instance of SurfaceParts with the updated container styles.
     */
    addContainerStyles(styles: ThemedSxProp<T>): SurfaceParts<T>;
}
declare namespace Surface {
    /**
     * Attributes for the Surface component.
     */
    type Attributes = {
        shape: Surface.Element<"shape">;
        corners?: Surface.Element<"corners">;
        border?: Surface.Element<"border">;
        background?: Surface.Element<"background">;
        foreground?: ColorPicker | Shade; // FIXME: What to do about foreground?
        shadow?: Surface.Element<"shadow">;
    };
    /**
     * Interface for a Surface element.
     */
    interface Element<E extends Surface.ElementType = Surface.ElementType> {
        readonly type: E;
        /**
         * Processes the SurfaceParts and element chain.
         *
         * @param parts - The current SurfaceParts.
         * @param chain - The chain of Surface elements.
         * @returns A tuple containing the updated SurfaceParts and element chain.
         */
        process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
            SurfaceParts<T>,
            Surface.Element[]
        ];
    }
    /**
     * Types of Surface elements.
     */
    type ElementType = Exclude<keyof Surface.Attributes, "foreground">;
    type ElementChain = Exclude<Surface.Attributes[Surface.ElementType], undefined>[];
    /**
     * State properties for the Surface component.
     */
    type StateProp = {
        state: "default" | "disabled" | "hovered" | "pressed";
    };
    /**
     * Props for the Surface component.
     */
    type Props = Partial<Surface.StateProp> & {
        children?: ReactNode;
        as?: AsProp;
        onHoverStart?: (e: SyntheticEvent) => void;
        onHoverEnd?: (e: SyntheticEvent) => void;
        onPress?: (e: SyntheticEvent) => void;
        onPressIn?: (e: SyntheticEvent) => void;
        onPressOut?: (e: SyntheticEvent) => void;
    };
    /**
     * Type for the Surface component with ref forwarding.
     */
    type ComponentType<P = {}> = TFC.WithRef<P & Surface.Props>;
}
declare const SurfaceSpec_base: import("@compositive/commons-spec").SpecClass<Surface.Attributes, Surface.StateProp>;
/**
 * A class representing the specification for a Surface component. It includes
 * attributes, state properties, and methods to convert the spec to a React
 * component.
 */
declare class SurfaceSpec<TVariants extends Surface.StateProp = Surface.StateProp> extends SurfaceSpec_base {
    /**
     * Converts the SurfaceSpec to a React component.
     *
     * @example
     *
     * ```tsx
     * const MySurface = surfaceSpec.toComponent();
     * <MySurface state="hovered">Content</MySurface>;
     * ```
     *
     * @returns The React component.
     */
    toComponent(): Surface.ComponentType<Partial<TVariants>>;
    /**
     * Gets the surface parts and foreground color based on the active variants.
     *
     * @param activeVariants - The active variants.
     * @returns An object containing the surface parts and foreground color.
     */
    private _getSurfaceParts;
}
declare const SolidFill_base: import("@compositive/commons-spec").SpecClass<{
    shade: Shade;
}, Record<never, string>>;
/**
 * A class representing a solid fill specification. Extends the Spec class with
 * a property for shade, and implements Surface.Element with type "background".
 *
 * @example
 *
 * ```typescript
 * const solidFill = new SolidFill({
 *   shade: theme.palette.primary.frontmost,
 * });
 * ```
 */
declare class SolidFill extends SolidFill_base implements Surface.Element<"background"> {
    readonly type = "background";
    /**
     * Processes the surface parts and chain to apply the solid fill style.
     *
     * @param parts - The surface parts to which the styles will be added.
     * @param chain - The chain of surface elements.
     * @returns A tuple containing the updated surface parts and chain.
     */
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
    /**
     * Generates the static style for the solid fill.
     *
     * @returns A StaticStyle instance with the background color.
     */
    private _getStyle;
}
declare const SolidBorder_base: import("@compositive/commons-spec").SpecClass<{
    width?: Length | undefined;
    shade: Shade;
}, Record<never, string>>;
/**
 * A class representing a solid border specification. Extends the Spec class
 * with properties for width and shade, and implements Surface.Element with type
 * "border".
 *
 * @example
 *
 * ```typescript
 * const solidBorder = new SolidBorder({
 *   width: "2px",
 *   shade: theme.palette.neutral.backmost,
 * });
 * ```
 */
declare class SolidBorder extends SolidBorder_base implements Surface.Element<"border"> {
    readonly type = "border";
    /**
     * Processes the surface parts and chain to apply the solid border style.
     *
     * @param parts - The surface parts to which the styles will be added.
     * @param chain - The chain of surface elements.
     * @returns A tuple containing the updated surface parts and chain.
     */
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
    /**
     * Generates the static style for the solid border.
     *
     * @returns A StaticStyle instance with the border width and color.
     */
    private _getStyle;
}
/**
 * Attributes for defining rounded corners.
 */
type RoundedCornersAttributes = {
    /**
     * The radius for all corners.
     */
    radius?: number;
    /**
     * The radius for the top-left corner.
     */
    topLeft?: number;
    /**
     * The radius for the top-right corner.
     */
    topRight?: number;
    /**
     * The radius for the bottom-left corner.
     */
    bottomLeft?: number;
    /**
     * The radius for the bottom-right corner.
     */
    bottomRight?: number;
};
declare const RoundedCorners_base: import("@compositive/commons-spec").SpecClass<RoundedCornersAttributes, Record<never, string>>;
/**
 * A class representing rounded corners specification. Extends the Spec class
 * with RoundedCornersAttributes and implements Surface.Element with type
 * "corners".
 *
 * @example
 *
 * ```typescript
 * const roundedCorners = new RoundedCorners({ radius: 10 });
 * ```
 */
declare class RoundedCorners extends RoundedCorners_base implements Surface.Element<"corners"> {
    readonly type = "corners";
    /**
     * Constructs a new RoundedCorners instance.
     *
     * @param radiusOrAttribs - Either a number representing the radius for all
     *   corners or an object with specific corner radii.
     * @param previousSpec - An optional previous RoundedCorners specification to
     *   extend.
     */
    constructor(radiusOrAttribs: RoundedCornersAttributes | number, previousSpec?: RoundedCorners);
    /**
     * Processes the surface parts and chain to apply the rounded corners style.
     *
     * @param parts - The surface parts to which the styles will be added.
     * @param chain - The chain of surface elements.
     * @returns A tuple containing the updated surface parts and chain.
     */
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
    /**
     * Generates the static style for the rounded corners.
     *
     * @returns A StaticStyle instance with the border radius.
     */
    // FIXME: Use the other attributes
    private _getStyle;
}
declare const DropShadow_base: import("@compositive/commons-spec").SpecClass<{
    elevation?: number | undefined;
    shade?: Shade | undefined;
}, Record<never, string>>;
/**
 * A class representing a DropShadow element for a Surface component.
 */
declare class DropShadow extends DropShadow_base implements Surface.Element<"shadow"> {
    readonly type = "shadow";
    /**
     * Processes the SurfaceParts and element chain to add shadow styles.
     *
     * @param parts - The current SurfaceParts.
     * @param chain - The chain of Surface elements.
     * @returns A tuple containing the updated SurfaceParts and element chain.
     */
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
}
/**
 * Attributes for the Rectangle shape.
 */
type RectangleAttributes = {
    padding?: number | {
        left?: number;
        right?: number;
        x?: number;
        top?: number;
        bottom?: number;
        y?: number;
    };
};
declare const Rectangle_base: import("@compositive/commons-spec").SpecClass<RectangleAttributes, Record<never, string>>;
/**
 * A class representing a Rectangle shape for a Surface component.
 */
declare class Rectangle extends Rectangle_base implements Surface.Element<"shape"> {
    readonly type = "shape";
    /**
     * Constructs a new Rectangle instance.
     *
     * @param attribs - The attributes for the Rectangle.
     * @param previousSpec - The previous Rectangle specification, if any.
     */
    constructor(attribs?: RectangleAttributes, previousSpec?: Rectangle);
    /**
     * Processes the SurfaceParts and element chain to add container styles.
     *
     * @param parts - The current SurfaceParts.
     * @param chain - The chain of Surface elements.
     * @returns A tuple containing the updated SurfaceParts and element chain.
     */
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
    /**
     * Gets the style properties for the Rectangle based on its attributes.
     *
     * @returns A StaticStyle instance with the computed styles.
     */
    private _getStyle;
}
/**
 * A hook that retrieves a Surface component from the theme based on the
 * provided names.
 *
 * This hook looks up the surface definitions in the theme using the provided
 * names. It returns the first matching Surface component it finds.
 *
 * @example
 *
 * ```tsx
 * const MySurface = useSurface(['button', 'pressable']);
 * <MySurface>Content</MySurface>
 * ```
 *
 * @param names - An array of names to look up in the theme.
 * @returns The Surface component.
 * @throws If no matching surface definition is found in the theme.
 */
declare function useSurface<T extends AnyTheme>(names: readonly (string | symbol)[]): Surface.ComponentType;
declare enum FontStyle {
    Normal = "normal",
    Italic = "italic",
    Oblique = "oblique"
}
type FontWeightMap = {
    Thin: 100;
    ExtraLight: 200;
    Light: 300;
    Regular: 400;
    Medium: 500;
    SemiBold: 600;
    Bold: 700;
    ExtraBold: 800;
    Black: 900;
};
declare class FontWeight<N extends keyof FontWeightMap = keyof FontWeightMap> {
    #private;
    static readonly Thin: FontWeight<"Thin">;
    static readonly ExtraLight: FontWeight<"ExtraLight">;
    static readonly Light: FontWeight<"Light">;
    static readonly Regular: FontWeight<"Regular">;
    static readonly Medium: FontWeight<"Medium">;
    static readonly SemiBold: FontWeight<"SemiBold">;
    static readonly Bold: FontWeight<"Bold">;
    static readonly ExtraBold: FontWeight<"ExtraBold">;
    static readonly Black: FontWeight<"Black">;
    static for(weight: number): FontWeight;
    readonly value: FontWeightMap[N];
    readonly name: N;
    private constructor();
    protected get index(): number;
    get bolder(): FontWeight | undefined;
    get lighter(): FontWeight | undefined;
    get fallbacks(): Readonly<FontWeight[]>;
    isLighterThan(other: FontWeight): boolean;
    isBolderThan(other: FontWeight): boolean;
    toString(): string;
}
declare class FontDesignation<W extends FontWeight = FontWeight, S extends FontStyle = FontStyle> {
    static fromFvd(fvd: string): FontDesignation<FontWeight<any>>;
    static for<W extends FontWeight<any>, S extends FontStyle>(weight: W, style: S): FontDesignation<W, S>;
    readonly weight: W;
    readonly style: S;
    private constructor();
    get fallbacks(): FontDesignation<any, any>[];
    get fvd(): string;
    get googleFontsDescription(): string;
    toString(): string;
    toHumanString(): string;
}
interface FontCatalog<F extends FontFamily> {
    getAll(): Promise<F[]>;
    get(name: string): Promise<F>;
}
interface FontLoader {
    loadMany(fonts: Iterable<Font>): Promise<void>;
    load(font: Font): Promise<void>;
    isLoaded(font: Font): boolean;
}
declare enum FontType {
    Cursive = "cursive",
    Fantasy = "fantasy",
    Monospace = "monospace",
    SansSerif = "sans-serif",
    Serif = "serif"
}
type WithThinFont = {
    readonly Thin: Font;
};
type WithThinItalicFont = {
    readonly ThinItalic: Font;
};
type WithThinObliqueFont = {
    readonly ThinOblique: Font;
};
type WithExtraLightFont = {
    readonly ExtraLight: Font;
};
type WithExtraLightItalicFont = {
    readonly ExtraLightItalic: Font;
};
type WithExtraLightObliqueFont = {
    readonly ExtraLightOblique: Font;
};
type WithLightFont = {
    readonly Light: Font;
};
type WithLightItalicFont = {
    readonly LightItalic: Font;
};
type WithLightObliqueFont = {
    readonly LightOblique: Font;
};
type WithRegularFont = {
    readonly Regular: Font;
};
type WithRegularItalicFont = {
    readonly RegularItalic: Font;
};
type WithRegularObliqueFont = {
    readonly RegularOblique: Font;
};
type WithMediumFont = {
    readonly Medium: Font;
};
type WithMediumItalicFont = {
    readonly MediumItalic: Font;
};
type WithMediumObliqueFont = {
    readonly MediumOblique: Font;
};
type WithSemiBoldItalicFont = {
    readonly SemiBoldItalic: Font;
};
type WithSemiBoldFont = {
    readonly SemiBold: Font;
};
type WithSemiBoldObliqueFont = {
    readonly SemiBoldOblique: Font;
};
type WithBoldItalicFont = {
    readonly BoldItalic: Font;
};
type WithBoldFont = {
    readonly Bold: Font;
};
type WithBoldObliqueFont = {
    readonly BoldOblique: Font;
};
type WithExtraBoldItalicFont = {
    readonly ExtraBoldItalic: Font;
};
type WithExtraBoldFont = {
    readonly ExtraBold: Font;
};
type WithExtraBoldObliqueFont = {
    readonly ExtraBoldOblique: Font;
};
type WithBlackFont = {
    readonly Black: Font;
};
type WithBlackItalicFont = {
    readonly BlackItalic: Font;
};
type WithBlackObliqueFont = {
    readonly BlackOblique: Font;
};
declare abstract class FontFamily {
    #private;
    readonly name: string;
    readonly fallback: (FontFamily | string)[];
    readonly fontType: FontType;
    constructor(name: string, fontType: FontType, fallback?: (FontFamily | string)[]);
    abstract get fontLoader(): FontLoader;
    abstract get fontCatalog(): FontCatalog<FontFamily>;
    withFont(w: FontWeight<"Thin">, s: FontStyle.Normal): WithThinFont & this;
    withFont(w: FontWeight<"Thin">, s: FontStyle.Italic): WithThinItalicFont & this;
    withFont(w: FontWeight<"Thin">, s: FontStyle.Oblique): WithThinObliqueFont & this;
    withFont(w: FontWeight<"ExtraLight">, s: FontStyle.Normal): WithExtraLightFont & this;
    withFont(w: FontWeight<"ExtraLight">, s: FontStyle.Italic): WithExtraLightItalicFont & this;
    withFont(w: FontWeight<"ExtraLight">, s: FontStyle.Oblique): WithExtraLightObliqueFont & this;
    withFont(w: FontWeight<"Light">, s: FontStyle.Normal): WithLightFont & this;
    withFont(w: FontWeight<"Light">, s: FontStyle.Italic): WithLightItalicFont & this;
    withFont(w: FontWeight<"Light">, s: FontStyle.Oblique): WithLightObliqueFont & this;
    withFont(w: FontWeight<"Regular">, s: FontStyle.Normal): WithRegularFont & this;
    withFont(w: FontWeight<"Regular">, s: FontStyle.Italic): WithRegularItalicFont & this;
    withFont(w: FontWeight<"Regular">, s: FontStyle.Oblique): WithRegularObliqueFont & this;
    withFont(w: FontWeight<"Medium">, s: FontStyle.Normal): WithMediumFont & this;
    withFont(w: FontWeight<"Medium">, s: FontStyle.Italic): WithMediumItalicFont & this;
    withFont(w: FontWeight<"Medium">, s: FontStyle.Oblique): WithMediumObliqueFont & this;
    withFont(w: FontWeight<"SemiBold">, s: FontStyle.Normal): WithSemiBoldFont & this;
    withFont(w: FontWeight<"SemiBold">, s: FontStyle.Italic): WithSemiBoldItalicFont & this;
    withFont(w: FontWeight<"SemiBold">, s: FontStyle.Oblique): WithSemiBoldObliqueFont & this;
    withFont(w: FontWeight<"Bold">, s: FontStyle.Normal): WithBoldFont & this;
    withFont(w: FontWeight<"Bold">, s: FontStyle.Italic): WithBoldItalicFont & this;
    withFont(w: FontWeight<"Bold">, s: FontStyle.Oblique): WithBoldObliqueFont & this;
    withFont(w: FontWeight<"ExtraBold">, s: FontStyle.Normal): WithExtraBoldFont & this;
    withFont(w: FontWeight<"ExtraBold">, s: FontStyle.Italic): WithExtraBoldItalicFont & this;
    withFont(w: FontWeight<"ExtraBold">, s: FontStyle.Oblique): WithExtraBoldObliqueFont & this;
    withFont(w: FontWeight<"Black">, s: FontStyle.Normal): WithBlackFont & this;
    withFont(w: FontWeight<"Black">, s: FontStyle.Italic): WithBlackItalicFont & this;
    withFont(w: FontWeight<"Black">, s: FontStyle.Oblique): WithBlackObliqueFont & this;
    /**
     * List of the fonts belonging to this family.
     */
    get fonts(): Readonly<Font[]>;
    get weights(): Readonly<FontWeight<any>[]>;
    get styles(): Readonly<FontStyle[]>;
    getFont(designation: FontDesignation): Font | undefined;
    getFont(weight: FontWeight, style: FontStyle): Font | undefined;
    getClosestFont(weight: FontWeight, style: FontStyle): Font | undefined;
    toString(): string;
}
declare enum TextTransform {
    None = "none",
    Uppercase = "uppercase",
    Lowercase = "lowercase",
    Capitalize = "capitalize"
}
declare class Font {
    readonly family: FontFamily;
    readonly weight: FontWeight;
    readonly style: FontStyle;
    readonly fontLoader: FontLoader;
    constructor(family: FontFamily, weight: FontWeight, style: FontStyle, fontLoader: FontLoader);
    withSize(size: AttributeValueDefinition<FontSpec["attributes"], "size">): FontSpec;
    with(attributeName: "size", value: AttributeValueDefinition<FontSpec["attributes"], "size">): FontSpec;
    with(values: Omit<AttributeDefinitions<FontSpec["attributes"]>, "font">): FontSpec;
    get designation(): FontDesignation;
    get bolder(): Font;
    get lighter(): Font;
    get italic(): Font;
    get normal(): Font;
    load(): Promise<void>;
    isLoaded(): boolean;
    toString(): string;
}
type FontSpecAttributes = {
    size: number;
    lineHeight?: number;
    font: Font;
    letterSpacing?: number;
    textTransform?: TextTransform;
    textAlign?: StyleProperties["textAlign"];
};
declare const FontSpec_base: import("@compositive/commons-spec").SpecClass<FontSpecAttributes, Record<never, string>>;
declare class FontSpec extends FontSpec_base implements ConvertibleToStyle<never> {
    get allFonts(): Font[];
    toStaticStyle(_theme?: UnknownTheme): StaticStyle;
}
declare function useTypography<T extends AnyTheme = DesignSystemThemes>(knownSpecNames: Theme.ReferenceCascade<T, "typography">): FontSpec;
declare function useTypography<_T extends AnyTheme = DesignSystemThemes>(specNames: readonly (string | symbol)[]): FontSpec | undefined;
/**
 * Abstract class representing a theme with attributes and variants.
 */
declare abstract class Theme<A extends Theme.Attributes, V extends Theme.Variants> {
    /**
     * Retrieves all fonts used in the theme. Uses memoization to cache the
     * result.
     */
    get allFonts(): Font[];
    /**
     * Loads all fonts used in the theme asynchronously.
     *
     * @example
     *
     * ```typescript
     * await theme.loadAllFonts();
     * ```
     */
    loadAllFonts(): Promise<void>;
    /**
     * Checks if all fonts used in the theme are loaded.
     *
     * @example
     *
     * ```typescript
     * const loaded = theme.areFontsLoaded();
     * ```
     */
    areFontsLoaded(): boolean;
    /**
     * Creates a new theme instance with the specified active variants.
     */
    abstract using(activeVariants: NullablePartial<V>): Theme<A, V>;
    /**
     * Returns the surfaces section of the theme.
     */
    abstract get surfaces(): A["surfaces"];
    /**
     * Gets the typography section of the theme.
     */
    abstract get typography(): A["typography"];
    /**
     * Gets the metrics section of the theme.
     */
    abstract get metrics(): A["metrics"];
    /**
     * Gets the palette section of the theme.
     */
    abstract get palette(): A["palette"];
    /**
     * Gets the color picks section of the theme.
     */
    abstract get colorPicks(): A["colorPicks"];
    /**
     * Gets the variant options of the theme.
     */
    abstract get variants(): Theme.VariantOptions<V>;
}
/**
 * Type accepting any theme ("top type" for themes, like `any`). It accepts any
 * theme and allows accessing any attribute.
 */
type AnyTheme = Theme<Theme.Open<Theme.Attributes>, Theme.Variants>;
/**
 * Type representing an uknown theme (equivalent of `unknown`, for themes). It
 * doesn't allow access to any attribute, except the default ones, and assumes
 * no variants.
 */
type UnknownTheme = Theme<Theme.Attributes, Theme.NoVariants>;
/**
 * Type representing a partial record with string or symbol keys.
 */
type PartialRecord<T> = Readonly<Record<string | symbol, T | undefined>>;
/**
 * Namespace containing types related to the Theme class.
 */
declare namespace Theme {
    /**
     * Attributes of a theme, including metrics, palette, color picks, surfaces,
     * and typography.
     */
    type Attributes = {
        readonly metrics: {
            readonly grid: number;
        };
        readonly palette: {
            readonly primary: ColorScale;
            readonly neutral: ColorScale;
            readonly info: ColorScale;
            readonly danger: ColorScale;
            readonly success: ColorScale;
            readonly warning: ColorScale;
        };
        readonly colorPicks: {
            readonly background: ColorPicker;
            readonly foreground: ColorPicker;
        };
        readonly surfaces: {
            readonly default: SurfaceSpec;
        };
        readonly typography: {
            readonly default: FontSpec;
        };
    };
    /**
     * Type representing attributes of a theme, allowing extending the sections
     * with more attributes.
     */
    type Open<A extends Attributes> = {
        readonly metrics: A["metrics"];
        readonly palette: A["palette"] & PartialRecord<ColorScale>;
        readonly colorPicks: A["colorPicks"] & PartialRecord<ColorPicker>;
        readonly surfaces: A["surfaces"] & PartialRecord<SurfaceSpec>;
        readonly typography: A["typography"] & PartialRecord<FontSpec>;
    };
    /**
     * Type representing extra keys for attributes.
     */
    type AttributeExtraKeys = Partial<Record<Exclude<keyof Attributes, "metrics">, string | symbol>>;
    /**
     * Type representing attributes with extra keys.
     */
    type AttributesWithExtraKeys<A extends Attributes, K extends AttributeExtraKeys> = {
        readonly [I in keyof A]: A[I] & (I extends keyof K ? undefined extends K[I] ? unknown : {
            readonly [J in K[I] & (string | symbol)]: NonNullable<A[I][keyof A[I]]>;
        } : unknown);
    };
    /**
     * Type representing only known attributes.
     */
    type OnlyKnownAttributes<T extends Attributes> = {
        readonly [K in keyof T]: {
            readonly [J in keyof T[K] as undefined extends T[K][J] ? never : J]: T[K][J];
        };
    };
    /**
     * Type representing theme variants.
     */
    type Variants = Readonly<Record<string, string>>;
    /**
     * Infers the variants of a given theme.
     */
    type InferVariants<T extends Theme<any, any>> = [
        T
    ] extends [
        never
    ] ? Theme.NoVariants : T extends Theme<any, infer V> ? V : Theme.NoVariants;
    /**
     * Type representing variant options for a theme.
     */
    type VariantOptions<V extends Theme.Variants> = {
        readonly [K in keyof V]: readonly V[K][];
    };
    /**
     * Type representing no variants.
     */
    type NoVariants = Record<never, never>;
    /**
     * Type representing a reference to a key in a theme.
     */
    type Reference<T extends AnyTheme, K extends keyof T> = T extends any ? keyof T[K] & (string | symbol) : never;
    /**
     * Type representing a reference cascade in a theme.
     */
    type ReferenceCascade<T extends AnyTheme, K extends keyof T> = T extends any ? readonly [
        ...(string | symbol)[],
        keyof T[K] & (string | symbol)
    ] | (keyof T[K] & (string | symbol)) : never;
}
/**
 * Interface keeping track of all the themes registered in the current design
 * system.
 *
 * It should be extended in the generated design system to have an entry for
 * each theme available at runtime.
 *
 * @example
 *
 * ```ts
 * type Id<T> = T;
 *
 * export interface RubikTheme extends Id<typeof Rubik> {}
 * export interface BlankTheme extends Id<typeof Blank> {}
 *
 * declare module "@compositive/foundation" {
 *   export interface AvailableThemes {
 *     Rubik: RubikTheme;
 *     Blank: BlankTheme;
 *   }
 * }
 * ```
 */
interface AvailableThemes {
}
/**
 * A type that resolves to `UnknownTheme` if `AvailableThemes` is empty,
 * otherwise it resolves to the values of `AvailableThemes`.
 */
type AvailableThemeValues = [
    keyof AvailableThemes
] extends [
    never
] ? UnknownTheme : AvailableThemes[keyof AvailableThemes];
// eslint-disable-next-line @typescript-eslint/naming-convention
declare const __delayedResolution: unique symbol;
/**
 * A hack to delay the resolution of {@link DesignSystemThemes}, otherwise it
 * gets eagerly evaluated as {@link UnknownTheme} at build time.
 */
type DelayedResolution = {
    __delayedResolution?: typeof __delayedResolution;
};
/**
 * A union of all the theme types currently defined in
 * {@linkcode AvailableThemes}.
 */
type DesignSystemThemes = AvailableThemeValues & DelayedResolution;
// FIXME: The previous type in the definition thunks for non-existing attributes should be "never" instead of "unknown"
// FIXME: Variant options should merge, not be replaced
/**
 * A {@link ComputedTheme} is a {@link Theme} implementation that allows for its
 * values to be derived through the use of pure functions and derivation rules.
 *
 * It's possible to create new themes from scratch, or through the extension of
 * other themes, by overriding only some of the values.
 */
declare class ComputedTheme<A extends Theme.Attributes, V extends Theme.Variants> extends Theme<A, V> {
    #private;
    /**
     * Creates a new ComputedTheme with the given definition.
     *
     * @example
     *
     * ```tsx
     * const theme = ComputedTheme.with({
     *   palette: {
     *     primary: new UniformLightnessColorScale("#0ff", 10),
     *     neutral: new UniformLightnessColorScale("#000", 10),
     *     info: new UniformLightnessColorScale("#00f", 10),
     *     warning: new UniformLightnessColorScale("#ff0", 10),
     *     danger: new UniformLightnessColorScale("#f00", 10),
     *     success: new UniformLightnessColorScale("#0f0", 10),
     *   },
     *
     *   colorPicks: {
     *     foreground: new ColorPicker((palette) => palette.frontmost),
     *     background: new ColorPicker((palette) => palette.backmost),
     *   },
     *
     *   typography: {
     *     default: HelveticaNeue.Regular.withSize(14),
     *   },
     *
     *   metrics: {
     *     grid: 4,
     *   },
     *
     *   surfaces: {
     *     default: new SurfaceSpec({ shape: new Rectangle({}) }),
     *   },
     * });
     * ```
     *
     * @param definition - The full attribute definition for the theme.
     * @returns A new ComputedTheme instance.
     */
    static with<D extends ComputedTheme.FullAttributeDefinition>(definition: D): ComputedTheme.InitiallyDefinedBy<D>;
    private constructor();
    /**
     * Returns a new Theme instance with the given active variants.
     *
     * @example
     *
     * ```tsx
     * const darkTheme = theme.using({ mode: "dark" });
     * ```
     *
     * @param activeVariants - The active variants to use.
     * @returns A new Theme instance.
     */
    using(activeVariants: NullablePartial<V>): Theme<A, V>;
    /**
     * Checks if the given object is equal to this ComputedTheme instance.
     *
     * @param other - The object to compare.
     * @returns `true` if the objects are equal, `false` otherwise.
     */
    [Equals](other: unknown): boolean;
    /**
     * Creates a new ComputedTheme with additional attribute definitions.
     *
     * @example
     *
     * ```tsx
     * const newTheme = computedTheme.with({ metrics: { grid: 8 } });
     * ```
     *
     * @param definition - The additional attribute definitions.
     * @returns A new ComputedTheme instance.
     */
    with<D extends ComputedTheme.OpenPartialAttributeDefinition<A>>(definition: D): ComputedTheme.WithExtraDefinition<A, D>;
    /**
     * Creates a new ComputedTheme with additional variant options.
     *
     * @example
     *
     * ```tsx
     * const newTheme = computedTheme.when("mode", {
     *   light: {},
     *   dark: {
     *     palette: {
     *       primary: (prev) => prev.reversed(),
     *       secondary: (prev) => prev.reversed(),
     *       neutral: (prev) => prev.reversed(),
     *       info: (prev) => prev.reversed(),
     *       warning: (prev) => prev.reversed(),
     *       danger: (prev) => prev.reversed(),
     *       success: (prev) => prev.reversed(),
     *     },
     *   },
     * });
     * ```
     *
     * @param variantName - The name of the variant.
     * @param options - The variant options.
     * @returns A new ComputedTheme instance.
     */
    when<N extends string, K extends string>(variantName: N, options: Readonly<Record<K, ComputedTheme.ClosedPartialAttributeDefinition<A>>>): ComputedTheme.WithExtraVariant<A, V, N, K>;
    get surfaces(): A["surfaces"];
    get typography(): A["typography"];
    get metrics(): A["metrics"];
    get palette(): A["palette"];
    get colorPicks(): A["colorPicks"];
    get variants(): Theme.VariantOptions<V>;
}
/**
 * Represents any computed theme.
 */
type AnyComputedTheme = ComputedTheme<Theme.Open<Theme.Attributes>, Theme.Variants>;
/**
 * Represents a computed theme with unknown attributes and no variants.
 */
type UnknownComputedTheme = ComputedTheme<Theme.Attributes, Theme.NoVariants>;
declare namespace ComputedTheme {
    /**
     * Represents a theme initially defined by a given definition.
     */
    type InitiallyDefinedBy<D extends OpenAttributeDefinition<any>> = WithExtraDefinition<Theme.Attributes, D>;
    /**
     * Represents a computed theme with extra attribute definitions.
     */
    type WithExtraDefinition<A extends Theme.Attributes, D extends OpenPartialAttributeDefinition<any>> = ComputedTheme<AttributesWithExtraDefinitions<A, D>, Theme.NoVariants>;
    /**
     * Represents a computed theme with extra variant options.
     */
    type WithExtraVariant<A extends Theme.Attributes, V extends Theme.Variants, N extends string, K extends string> = ComputedTheme<A, Merge<V, {
        readonly [I in N]: K;
    }>>;
    /**
     * Represents open partial attributes.
     */
    type OpenPartialAttributes = {
        readonly [K in keyof Theme.Attributes]?: Readonly<Record<string | symbol, Theme.Attributes[K][keyof Theme.Attributes[K]] | undefined>>;
    };
    /**
     * Represents closed partial attributes.
     */
    type ClosedPartialAttributes<A extends Theme.Attributes> = {
        readonly [K in keyof A & (string | symbol)]?: {
            readonly [J in keyof A[K] & (string | symbol)]?: A[K][J];
        };
    };
    /**
     * Represents a full attribute definition.
     */
    type FullAttributeDefinition = OpenAttributeDefinition<{}>;
    /**
     * An attribute definition object, derived from a parent definition {@link P}.
     *
     * As an "open" definition (as opposed to a
     * {@link ClosedPartialAttributeDefinition}), it is allowed to define
     * additional properties to the attributes compared to the parent.
     *
     * @template P Parent attributes
     * @see {@link Theme.Open}
     */
    type OpenPartialAttributeDefinition<P extends Theme.Attributes> = Evaluator.AttributeDefinition<1, OpenPartialAttributes, P, Theme.Open<P>>;
    /**
     * Represents a closed partial attribute definition.
     */
    type ClosedPartialAttributeDefinition<P extends Theme.Attributes> = ChainEvaluator.PartialAttributeDefinition<1, P, P>;
    /**
     * Represents an open attribute definition.
     */
    type OpenAttributeDefinition<P extends Evaluator.AttributeTypeRecord<1>> = Evaluator.AttributeDefinition<1, Theme.Open<Theme.Attributes>, P, Theme.Open<Theme.Attributes>>;
    /**
     * Represents the defined keys of an attribute type record.
     */
    type DefinedKeys<D extends Evaluator.AttributeTypeRecord<1>> = {
        [I in keyof D]: keyof D[I] & (string | symbol);
    };
    /**
     * Represents attributes with extra definitions.
     */
    type AttributesWithExtraDefinitions<A extends Theme.Attributes, D extends Evaluator.AttributeTypeRecord<1>> = Theme.AttributesWithExtraKeys<A, DefinedKeys<D>>;
}
/**
 * Generates a length value based on the theme's grid metric and a multiplier.
 *
 * @example
 *
 * ```tsx
 * // If the currently active theme has a `metrics.grid` value of 8px, the padding for the component will be 16px
 * const padding = new Style({ padding: units(2) });
 * ```
 *
 * @param multiplier - A number to multiply the theme's grid metric by. Defaults
 *   to 1.
 * @returns A function that takes a theme and returns a length value as a
 *   string.
 */
declare const units: (multiplier?: number) => <T extends AnyTheme>(theme: T) => Length;
/**
 * Shorthand for the {@linkcode units()} function.
 */
declare const u: (multiplier?: number) => <T extends AnyTheme>(theme: T) => Length;
declare const currentTheme: unique symbol;
/**
 * Type representing the current theme extracted from the props.
 */
type CurrentTheme<P extends TFC.PropsWithTheme> = NonNullable<P[typeof currentTheme]>;
declare namespace TFC {
    /**
     * Interface for props that include an optional theme.
     */
    interface PropsWithTheme {
        readonly [currentTheme]?: UnknownTheme;
    }
    /**
     * Common properties for themed components.
     */
    type CommonThemedProps<T extends UnknownTheme = DesignSystemThemes> = {
        sx?: ThemedSxProp<T>;
    };
    /**
     * Type that binds props with a specific theme, omitting the current theme
     * symbol.
     */
    type BindPropsWithTheme<P extends Record<string, any>, T extends UnknownTheme> = Omit<P & {
        [currentTheme]: T;
    }, typeof currentTheme>;
    /**
     * Type for a render function with a ref, supporting theming.
     */
    type RenderFnWithRef<P extends Record<string, any>, E = HTMLElement> = <T extends UnknownTheme = DesignSystemThemes>(props: TFC.BindPropsWithTheme<P, T> & TFC.CommonThemedProps<T>, ref: Ref<E>) => ReactElement;
    /**
     * Type for a component with a ref, supporting theming.
     */
    type WithRef<P extends Record<string, any>, E = HTMLElement> = TFC<P & RefAttributes<E>>;
}
/**
 * A Themed Functional Component.
 *
 * It provides the same signature as a regular React Functional Component, but
 * with appropriate bindings for the `sx` property with the current themes.
 */
interface TFC<P extends Record<string, any> = {}> {
    <T extends UnknownTheme = DesignSystemThemes>(props: TFC.BindPropsWithTheme<P, T> & TFC.CommonThemedProps<T>): ReactElement;
    displayName?: string;
}
/**
 * Utility for creating a themed functional component with a ref.
 *
 * @example
 *
 * ```tsx
 * const ThemedComponent = TFC.forwardRef((props, ref) => {
 *   return <View ref={ref} sx={props.sx}>Hello, themed world!</div>;
 * });
 * ```
 */
declare const TFC: {
    readonly forwardRef: <P extends Record<string, any> = {}, E = HTMLElement>(render: TFC.RenderFnWithRef<P, E>) => TFC.WithRef<P, E>;
};
declare namespace ThemeConsumer {
    /**
     * Props for the ThemeConsumer component.
     */
    type Props<T extends UnknownTheme> = {
        /**
         * A function that receives the current theme and returns a ReactNode.
         */
        children: (value: T) => ReactNode;
    };
}
/**
 * A component that consumes the theme context and passes the theme value to its
 * children.
 *
 * @example
 *
 * ```tsx
 * <ThemeConsumer>
 *   {(theme) => (
 *     <div style={{ color: theme.palette.primary.darkest }}>Hello</div>
 *   )}
 * </ThemeConsumer>;
 * ```
 */
declare const ThemeConsumer: <T extends UnknownTheme = DesignSystemThemes>({ children }: ThemeConsumer.Props<T>) => ReactElement;
declare namespace ThemeProvider {
    /**
     * Props for the ThemeProvider component.
     */
    type Props<T extends UnknownTheme> = NullablePartial<Theme.InferVariants<T>> & {
        /**
         * The theme object to be used.
         */
        theme: T;
        /**
         * The children elements to be rendered within the provider.
         */
        children: ReactNode;
    };
}
/**
 * A provider component that supplies a theme to its children. It's possible to
 * specify which theme variants should be evaluated as active in the current
 * tree.
 *
 * @example
 *
 * ```tsx
 * <ThemeProvider theme={Rubik} mode="dark">
 *   <MyComponent />
 * </ThemeProvider>;
 * ```
 */
declare const ThemeProvider: <T extends UnknownTheme>({ theme, children, ...variants }: ThemeProvider.Props<T>) => ReactElement;
/**
 * A custom hook that retrieves the current theme from the ThemeContext. It
 * asserts that the theme is defined and throws an error if it is not.
 *
 * @example
 *
 * ```tsx
 * const theme = useTheme();
 * console.log(theme);
 * ```
 */
declare const useTheme: <T extends UnknownTheme = DesignSystemThemes>() => T;
/**
 * Represents style properties that can be themed. Each property can either be a
 * static value or a function that takes a theme and returns a value.
 */
type ThemedStyleProperties<T extends UnknownTheme> = {
    [X in keyof StyleProperties]: StyleProperties[X] | ((theme: T) => StyleProperties[X]);
};
/**
 * Represents a generator for themed style properties. It can either be a
 * {@linkcode ThemedStyleProperties} object or a function that returns a
 * ThemedStyleProperties object.
 */
type ThemedStylePropertiesGenerator<T extends UnknownTheme> = ThemedStyleProperties<T> | ((theme: T) => ThemedStyleProperties<T>);
/**
 * Creates a theme-specific style processor function.
 *
 * This function takes a theme and returns another function that processes
 * themed style properties into either a StaticStyle or StyleProperties.
 *
 * @example
 *
 * ```typescript
 * const theme = { colors: { primary: "blue" } };
 * const processor = createThemeSxProcessor(theme);
 * const style = processor({ color: "primary" });
 * ```
 *
 * @param theme - The theme to use for resolving style properties.
 * @returns A function that processes themed style properties.
 */
/* #__NO_SIDE_EFFECTS__ */
declare const createThemeSxProcessor: <T extends UnknownTheme>(theme: T) => (style: ThemedStyleProperties<T>) => StaticStyle | StyleProperties;
/**
 * Resolves themed style properties into static style properties.
 *
 * This function takes themed style properties and a theme, and resolves them
 * into static style properties by applying the theme to any functions within
 * the style properties.
 *
 * @param themedStyleProperties - The themed style properties or generator
 *   function.
 * @param theme - The theme to use for resolving style properties.
 * @returns The resolved static style properties.
 * @internal
 */
declare const resolveThemedStyleProperties: <T extends UnknownTheme>(themedStyleProperties: ThemedStylePropertiesGenerator<T>, theme: T) => StyleProperties;
/**
 * Checks if a value is convertible to a StaticStyle.
 *
 * @example
 *
 * ```typescript
 * const value = { toStaticStyle: (theme) => new StaticStyle() };
 * const result = isConvertibleToStyle(value); // true
 * ```
 *
 * @param value - The value to check.
 * @returns True if the value is convertible to a StaticStyle, otherwise false.
 */
declare const isConvertibleToStyle: <T extends UnknownTheme>(value: unknown) => value is ConvertibleToStyle<T>;
/**
 * Interface for objects that can be converted to a StaticStyle.
 */
interface ConvertibleToStyle<T extends UnknownTheme> {
    /**
     * Converts the object to a StaticStyle using the provided theme.
     *
     * @param theme - The theme to use for conversion.
     * @returns The resulting StaticStyle.
     */
    toStaticStyle(theme: T): StaticStyle;
}
/**
 * Represents a themed style property item. It can be a ConvertibleToStyle,
 * StaticStyle, StyleProperties, or ThemedStyleProperties.
 */
type ThemedSxPropItem<T extends UnknownTheme> = ConvertibleToStyle<T> | StaticStyle | StyleProperties | ThemedStyleProperties<T>;
/**
 * Represents a themed SxProp.
 */
type ThemedSxProp<T extends UnknownTheme> = SxProp<ThemedSxPropItem<T>>;
declare class ThemedStyle<T extends UnknownTheme> implements ConvertibleToStyle<T> {
    #private;
    // TODO: Copying this static method here is not the best API design decision.
    // It would be better to find a better structure to separate "Style" manipulation from
    // the "SxProp" manipulation
    static concat<T extends UnknownTheme>(...sxProps: ThemedSxProp<T>[]): ThemedSxProp<T>;
    constructor(themedStyleProperties: ThemedStylePropertiesGenerator<T>);
    toStaticStyle(theme: T): StaticStyle;
}
declare module ThemedStyleWrapper {
    export { ThemedStyle };
}
import Style$0 = ThemedStyleWrapper.ThemedStyle;
declare module StyleWrapper {
    export { Style$0 as Style };
}
import ThemedStyle$0 = StyleWrapper.Style;
/**
 * Hook that creates a {@linkcode StaticStyle} from style properties.
 *
 * This hook takes a function that returns style properties and a dependency
 * list. The resulting style is memoized, and recreated whenever the
 * dependencies change.
 *
 * @example
 *
 * ```typescript
 * const style = useStaticStyle(
 *   () => ({ color: foregroundColor }),
 *   [foregroundColor],
 * );
 * ```
 *
 * @param rules - A function that returns style properties.
 * @param deps - A list of dependencies that, when changed, will cause the
 *   StaticStyle to be re-created.
 * @returns A StaticStyle instance.
 */
declare const useStaticStyle: (rules: () => StyleProperties, deps: DependencyList) => StaticStyle;
/**
 * A hook that creates a ThemedStyle from themed style properties.
 *
 * This hook takes a function that returns themed style properties and a
 * dependency list. The resulting style is cached, and recreated only when one
 * of the dependencies change.
 *
 * @example
 *
 * ```typescript
 * const themedStyle = useThemedStyle(
 *   () => ({ width: (theme) => theme.grid.unit * size }),
 *   [size],
 * );
 * ```
 *
 * @param rules - A function that returns themed style properties.
 * @param deps - A list of dependencies that, when changed, will cause the
 *   ThemedStyle to be re-created.
 * @returns A ThemedStyle instance.
 */
declare const useThemedStyle: <T extends UnknownTheme = DesignSystemThemes>(rules: () => ThemedStyleProperties<T>, deps: DependencyList) => ThemedStyle$0<T>;
interface View {
    <T extends UnknownTheme>(props: RefAttributes<HTMLElement> & ViewProps<ThemedSxPropItem<T>>): JSX.Element;
}
declare namespace View {
    type Props<T extends UnknownTheme> = ViewProps<ThemedSxPropItem<T>>;
}
declare const View: View;
interface Text {
    <T extends UnknownTheme>(props: RefAttributes<HTMLElement> & TextProps<ThemedSxPropItem<T>>): JSX.Element;
}
declare namespace Text {
    type Props<T extends UnknownTheme> = TextProps<ThemedSxPropItem<T>>;
}
declare const Text: Text;
interface Image {
    <T extends UnknownTheme>(props: ImageProps<ThemedSxPropItem<T>> & RefAttributes<HTMLImageElement>): JSX.Element;
}
declare namespace Image {
    type Props<T extends UnknownTheme> = ImageProps<ThemedSxPropItem<T>>;
}
declare const Image: Image;
interface Input {
    <T extends UnknownTheme>(props: InputProps<ThemedSxPropItem<T>>): JSX.Element;
}
declare namespace Input {
    type Props<T extends UnknownTheme> = InputProps<ThemedSxPropItem<T>>;
}
declare const Input: Input;
interface Upload {
    <T extends UnknownTheme>(props: RefAttributes<HTMLInputElement> & UploadProps<ThemedSxPropItem<T>>): JSX.Element;
}
declare namespace Upload {
    type Props<T extends UnknownTheme> = UploadProps<ThemedSxPropItem<T>>;
}
declare const Upload: Upload;
declare class StaticStyle extends Style {
}
/**
 * Type representing a hexadecimal color string.
 */
type HexColorString = `#${string}`;
/**
 * Predicate to check if a string is a valid hex color string.
 *
 * @example
 *
 * ```ts
 * isHexColorString("#ff5733"); // true
 * isHexColorString("#f53"); // true
 * isHexColorString("123456"); // false
 * ```
 */
declare const isHexColorString: Predicate<HexColorString>;
/**
 * Parses a hex color string to a ColorRGB object.
 *
 * @example
 *
 * ```ts
 * const rgb = parseHexColorString("#ff5733");
 * // rgb: { r: 255, g: 87, b: 51, a: 1 }
 * ```
 *
 * @param color - The hex color string to parse.
 * @returns An object representing the color in RGB format.
 */
declare function parseHexColorString(color: HexColorString): ColorRGB;
/**
 * Converts a ColorRGB object to a hex color string.
 *
 * @example
 *
 * ```ts
 * const hex = toHexColorString({ r: 255, g: 87, b: 51, a: 1 });
 * // hex: "#ff5733"
 * ```
 *
 * @param rgb - The RGB color object to convert.
 * @returns The hex color string.
 */
declare function toHexColorString(rgb: ColorRGB): HexColorString;
/**
 * Class representing a color and its various transformations.
 */
declare class Color implements ConvertibleToStyle<never> {
    #private;
    /**
     * Creates a Color instance from a binary representation.
     *
     * @param binary - The binary representation of the color.
     * @returns A new Color instance.
     */
    static fromBinary([r, g, b]: Uint8Array): Color;
    /**
     * Creates an instance of Color.
     *
     * @param colorValue - The value representing the color.
     */
    constructor(colorValue: ColorLikeValue);
    /**
     * Returns a style that uses the color as foreground.
     *
     * @returns The static style for the foreground color.
     */
    asForeground(): StaticStyle;
    /**
     * Returns a style that uses the color as background.
     *
     * @returns The static style for the background color.
     */
    asBackground(): StaticStyle;
    /**
     * Converts the color to a static style that uses it as the foreground.
     *
     * @returns The static style for the color.
     * @see this#asForeground()
     */
    toStaticStyle(): StaticStyle;
    /**
     * Gets the hexadecimal representation of the color.
     *
     * @returns The hexadecimal color string.
     */
    get hex(): HexColorString;
    /**
     * Gets the red component of the color.
     *
     * @returns The red component value.
     */
    get r(): number;
    /**
     * Gets the green component of the color.
     *
     * @returns The green component value.
     */
    get g(): number;
    /**
     * Gets the blue component of the color.
     *
     * @returns The blue component value.
     */
    get b(): number;
    get h(): number;
    get s(): number;
    get l(): number;
    /**
     * Gets the alpha (transparency) component of the color.
     *
     * @returns The alpha component value.
     */
    get a(): number;
    /**
     * Gets the RGB representation of the color.
     *
     * @returns An array containing the RGB values.
     */
    get rgb(): [
        red: number,
        green: number,
        blue: number
    ];
    /**
     * Gets the HSL representation of the color.
     *
     * @returns An array containing the HSL values.
     */
    get hsl(): [
        hue: number,
        saturation: number,
        lightness: number
    ];
    alpha(alpha: number): Color;
    toString(): string;
    toBinary(): Uint8Array;
}
type ColorLikeValue = Color | HexColorString | PrimitiveColor;
/**
 * Representation of a color part of a certain color scale
 */
declare class Shade extends Color {
    readonly colorScale: ColorScale;
    readonly index: number;
    /**
     * Creates an instance of Shade.
     *
     * @param colorScale - The color scale to which this shade belongs.
     * @param index - The index of the shade in the color scale.
     * @param colorValue - The color value of the shade.
     */
    constructor(colorScale: ColorScale, index: number, colorValue: ColorLikeValue);
    /**
     * Transposes this shade onto another color scale at the same index.
     *
     * @example
     *
     * ```ts
     * const transposedShade = shade.transposeOn(otherScale);
     * ```
     *
     * @param otherScale - The color scale to transpose onto.
     * @returns The shade from the other scale at the same index, or undefined if
     *   out of bounds.
     */
    transposeOn(otherScale: ColorScale): Shade | undefined;
    /**
     * Moves backward by the given number of steps in the color scale.
     *
     * @example
     *
     * ```ts
     * const newShade = shade.backward(1, "clamp");
     * ```
     *
     * @param steps - The number of steps to move backward.
     * @param outOfBoundariesMethod - The method to handle out-of-boundaries
     *   indices.
     * @returns The shade at the new index, or undefined if out of bounds.
     */
    backward(steps?: number, outOfBoundariesMethod?: "none"): Shade | undefined;
    backward(steps: number, outOfBoundariesMethod: "clamp" | "wrap"): Shade;
    /**
     * Moves forward by the given number of steps in the color scale.
     *
     * @example
     *
     * ```ts
     * const newShade = shade.frontward(2, "wrap");
     * ```
     *
     * @param steps - The number of steps to move forward.
     * @param outOfBoundariesMethod - The method to handle out-of-boundaries
     *   indices.
     * @returns The shade at the new index, or undefined if out of bounds.
     */
    frontward(steps?: number, outOfBoundariesMethod?: "none"): Shade | undefined;
    frontward(steps: number, outOfBoundariesMethod: "clamp" | "wrap"): Shade;
    /**
     * Checks if another object is equal to this shade.
     *
     * @param other - The object to compare with.
     * @returns True if the objects are equal, otherwise false.
     */
    [Equals](other: unknown): boolean;
}
declare namespace ColorScale {
    /**
     * Defines the methods for handling out-of-bounds indices.
     *
     * - "clamp": Limits the index to the range.
     * - "none": Returns undefined for out-of-bounds indices.
     * - "wrap": Considers the range circular and "wraps" the index around. Negative
     *   indices select values backward from the end and indices higher than the
     *   upper bound select values from the beginning of the range
     */
    type OutOfBoundsMethod = "clamp" | "none" | "wrap";
}
declare namespace ColorScale {
    /**
     * Type interface for ColorScale, implementing the Higher-Kinded Type (HKT)
     * pattern.
     *
     * @internal
     */
    interface Type extends HKT {
        readonly type: FixedColorScale;
    }
}
/**
 * Abstract base class representing a color scale.
 */
declare abstract class ColorScale extends Sequence<Shade> implements Indexed<number, Shade | undefined>, HKT.Holder {
    hktForSameElementKind: ColorScale.Type;
    /**
     * Constructs a new instance of the ColorScale class.
     */
    constructor();
    /**
     * Retrieves the shade at the specified index using the given out-of-bounds
     * method.
     *
     * @example
     *
     * ```ts
     * colorScale.get(2, "clamp");
     * ```
     *
     * @param key - The index of the shade.
     * @param outOfBoundsMethod An optional parameter that specifies how to handle
     *   out-of-bounds indices. If "none" (default), returns undefined for
     *   out-of-bounds indices. If "clamp", returns the nearest valid index. If
     *   "wrap", wraps around to the opposite end of the scale.
     * @returns The shade at the specified index or undefined.
     */
    get(key: number, outOfBoundsMethod?: "none"): Shade | undefined;
    get(key: number, outOfBoundsMethod: "clamp" | "wrap"): Shade;
    get(key: number, outOfBoundsMethod: ColorScale.OutOfBoundsMethod): Shade | undefined;
    /**
     * Checks if the shade at the specified index exists.
     *
     * @example
     *
     * ```ts
     * colorScale.has(1); // true or false
     * ```
     *
     * @param key - The index to check.
     * @returns `true` if the shade exists, otherwise `false`.
     */
    has(key: number): boolean;
    /**
     * Gets the number of shades in the color scale.
     *
     * @returns The length of the color scale.
     */
    get length(): number;
    /**
     * Gets the backmost (first) shade in the color scale.
     *
     * @returns The backmost shade.
     */
    get backmost(): Shade;
    /**
     * Gets the midpoint shade in the color scale.
     *
     * @returns The midpoint shade.
     */
    get midpoint(): Shade;
    /**
     * Gets the frontmost (last) shade in the color scale.
     *
     * @returns The frontmost shade.
     */
    get frontmost(): Shade;
    /**
     * Filters the shades in the color scale based on a callback function.
     *
     * @example
     *
     * ```ts
     * colorScale.filter((shade) => shade.value > 50);
     * ```
     *
     * @param cb - The callback function to determine if a shade should be
     *   included.
     * @returns A new ColorScale with the filtered shades.
     */
    filter(cb: (value: Shade, iteration: number) => boolean): ColorScale;
    /**
     * Compares the current instance with another object for equality.
     *
     * @param other The object to compare with.
     * @returns True if the objects are equal, false otherwise.
     */
    [Equals](other: unknown): boolean;
    /**
     * Returns the shades in the color scale.
     */
    abstract get shades(): Shade[];
    /**
     * Gets a reversed version of color scale, with the shades in opposite order.
     *
     * @returns A new ColorScale with the shades in reverse order.
     */
    abstract reversed(): ColorScale;
    /**
     * Builds the next color scale with the same element kind.
     *
     * @param source - The source iterable or iterator of shades.
     * @returns A new ColorScale.
     * @internal
     */
    protected buildNextWithSameElementKind<H extends HKT | HKT.Holder>(source: Iterable<Shade> | (() => Iterator<Shade>)): HKT.Kind<H, [
        Shade
    ]>;
}
/**
 * A color scale with a statically defined set of shades.
 */
declare class FixedColorScale extends ColorScale {
    /**
     * An array of `Shade` objects representing the shades in the color scale.
     */
    readonly shades: Shade[];
    /**
     * Constructs a new `FixedColorScale` instance.
     *
     * @param colors - An iterable of color values or a function that returns an
     *   iterator of color values.
     */
    constructor(colors: Iterable<ColorLikeValue> | (() => Iterator<ColorLikeValue>));
    /**
     * Gets the reversed color scale.
     *
     * @example
     *
     * ```ts
     * const reversedScale = colorScale.reversed();
     * ```
     *
     * @returns A new FixedColorScale with the shades in reverse order.
     */
    reversed(): FixedColorScale;
}
/**
 * Class representing a color picker that selects a shade from a color scale.
 */
declare class ColorPicker {
    #private;
    /**
     * Creates an instance of ColorPicker.
     *
     * @param colorPickingFn - The function used to pick a color from the color
     *   scale.
     */
    constructor(colorPickingFn: ColorPicker.ColorPickingFunction);
    /**
     * Picks a shade from the given color scale, optionally considering a
     * background shade.
     *
     * @example
     *
     * ```ts
     * const pickedShade = colorPicker.pickColor(colorScale, backgroundShade);
     * ```
     *
     * @param palette - The color scale to pick a shade from.
     * @param background - An optional background shade that might influence the
     *   picked color.
     * @returns The picked shade.
     */
    pickColor(palette: ColorScale, background?: Shade): Shade;
}
declare namespace ColorPicker {
    /**
     * Type representing a function that picks a shade from a color scale.
     */
    type ColorPickingFunction = (palette: ColorScale, background?: Shade) => Shade;
}
declare namespace ColorContext {
    /**
     * Represents a color selection which can be a ColorPicker, Shade, or
     * Theme.ReferenceCascade.
     */
    type ColorPick<T extends UnknownTheme = DesignSystemThemes> = ColorPicker | Shade | Theme.ReferenceCascade<T, "colorPicks">;
    /**
     * Defines the output structure with foreground, background, and palette
     * shades.
     */
    type Output = {
        foreground: Shade;
        background: Shade;
        palette: ColorScale;
    };
    /**
     * Defines the input structure for ColorContext with foreground, background,
     * and palette color picks.
     */
    type Input<T extends UnknownTheme = DesignSystemThemes> = {
        foreground: ColorPick<T>;
        background: ColorPick<T>;
        palette: Theme.Reference<T, "palette">;
    };
}
declare namespace ColorProvider {
    /**
     * Props for the ColorProvider component, including optional foreground,
     * background, and palette, as well as the children elements.
     */
    type Props<T extends UnknownTheme = DesignSystemThemes> = Partial<ColorContext.Input<T>> & {
        children: ReactNode;
    };
}
/**
 * A provider component that supplies color context values to its children. The
 * values passed can be references to the `colorPicks` section of the theme,
 * {@link Shade|shades} or {@link ColorPicker|color-picking objects}.
 *
 * In all cases the values will be resolved to color shades before being passed
 * to the child components.
 *
 * @example
 *
 * ```tsx
 * <ColorProvider
 *   foreground={theme.palette.neutral.frontmost}
 *   background="background"
 *   palette="neutral"
 * >
 *   <YourComponent />
 * </ColorProvider>;
 * ```
 */
declare const ColorProvider: <T extends UnknownTheme = DesignSystemThemes>(props: ColorProvider.Props<T>) => ReactElement;
declare namespace ColorConsumer {
    /**
     * Props for the ColorConsumer component, which includes a function as
     * children that receives the current color context values.
     */
    type Props = {
        children: (values: ColorContext.Output) => ReactElement;
    };
}
/**
 * A consumer component that receives the current color context values and
 * passes them to its children function.
 *
 * @example
 *
 * ```tsx
 * <ColorConsumer>
 *   {({ foreground, background, palette }) => (
 *     <div style={{ color: foreground }}>
 *       Background color is {background.hex} and the palette has{" "}
 *       {palette.length} shades.
 *     </div>
 *   )}
 * </ColorConsumer>;
 * ```
 */
declare const ColorConsumer: ({ children }: ColorConsumer.Props) => ReactElement;
declare const useCurrentForeground: () => Shade;
declare const useCurrentBackground: () => Shade;
declare const useCurrentPalette: () => ColorScale;
declare const transparent: Color;
// Red colors
declare const mediumVioletRed: Color;
declare const deepPink: Color;
declare const paleVioletRed: Color;
declare const hotPink: Color;
declare const lightPink: Color;
declare const pink: Color;
declare const darkRed: Color;
declare const red: Color;
declare const firebrick: Color;
declare const crimson: Color;
declare const indianRed: Color;
declare const lightCoral: Color;
declare const salmon: Color;
declare const darkSalmon: Color;
declare const lightSalmon: Color;
// Orange colors
declare const orangeRed: Color;
declare const tomato: Color;
declare const darkOrange: Color;
declare const coral: Color;
declare const orange: Color;
// Yellow colors
declare const darkKhaki: Color;
declare const gold: Color;
declare const khaki: Color;
declare const peachPuff: Color;
declare const yellow: Color;
declare const paleGoldenrod: Color;
declare const moccasin: Color;
declare const papayaWhip: Color;
declare const lightGoldenrodYellow: Color;
declare const lemonChiffon: Color;
declare const lightYellow: Color;
// Brown colors
declare const maroon: Color;
declare const brown: Color;
declare const saddleBrown: Color;
declare const sienna: Color;
declare const chocolate: Color;
declare const darkGoldenrod: Color;
declare const peru: Color;
declare const rosyBrown: Color;
declare const goldenrod: Color;
declare const sandyBrown: Color;
declare const tan: Color;
declare const burlywood: Color;
declare const wheat: Color;
declare const navajoWhite: Color;
declare const bisque: Color;
declare const blanchedAlmond: Color;
declare const cornsilk: Color;
// Green colors
declare const darkGreen: Color;
declare const green: Color;
declare const darkOliveGreen: Color;
declare const forestGreen: Color;
declare const seaGreen: Color;
declare const olive: Color;
declare const oliveDrab: Color;
declare const mediumSeaGreen: Color;
declare const limeGreen: Color;
declare const lime: Color;
declare const springGreen: Color;
declare const mediumSpringGreen: Color;
declare const darkSeaGreen: Color;
declare const mediumAquamarine: Color;
declare const yellowGreen: Color;
declare const lawnGreen: Color;
declare const chartreuse: Color;
declare const lightGreen: Color;
declare const greenYellow: Color;
declare const paleGreen: Color;
// Cyan colors
declare const teal: Color;
declare const darkCyan: Color;
declare const lightSeaGreen: Color;
declare const cadetBlue: Color;
declare const darkTurquoise: Color;
declare const mediumTurquoise: Color;
declare const turquoise: Color;
declare const aqua: Color;
declare const cyan: Color;
declare const aquamarine: Color;
declare const paleTurquoise: Color;
declare const lightCyan: Color;
// Blue colors
declare const navy: Color;
declare const darkBlue: Color;
declare const mediumBlue: Color;
declare const blue: Color;
declare const midnightBlue: Color;
declare const royalBlue: Color;
declare const steelBlue: Color;
declare const dodgerBlue: Color;
declare const deepSkyBlue: Color;
declare const cornflowerBlue: Color;
declare const skyBlue: Color;
declare const lightSkyBlue: Color;
declare const lightSteelBlue: Color;
declare const lightBlue: Color;
declare const powderBlue: Color;
// Purple, violet, and magenta colors
declare const indigo: Color;
declare const purple: Color;
declare const darkMagenta: Color;
declare const darkViolet: Color;
declare const darkSlateBlue: Color;
declare const blueViolet: Color;
declare const darkOrchid: Color;
declare const fuchsia: Color;
declare const magenta: Color;
declare const slateBlue: Color;
declare const mediumSlateBlue: Color;
declare const mediumOrchid: Color;
declare const mediumPurple: Color;
declare const orchid: Color;
declare const violet: Color;
declare const plum: Color;
declare const thistle: Color;
declare const lavender: Color;
// White colors
declare const mistyRose: Color;
declare const antiqueWhite: Color;
declare const linen: Color;
declare const beige: Color;
declare const whiteSmoke: Color;
declare const lavenderBlush: Color;
declare const oldLace: Color;
declare const aliceBlue: Color;
declare const seashell: Color;
declare const ghostWhite: Color;
declare const honeydew: Color;
declare const floralWhite: Color;
declare const azure: Color;
declare const mintCream: Color;
declare const snow: Color;
declare const ivory: Color;
declare const white: Color;
// Gray and black colors
declare const black: Color;
declare const darkSlateGray: Color;
declare const dimGray: Color;
declare const slateGray: Color;
declare const gray: Color;
declare const lightSlateGray: Color;
declare const darkGray: Color;
declare const silver: Color;
declare const lightGray: Color;
declare const gainsboro: Color;
declare enum ContrastLevel {
    /**
     * Less than WCAG 3:1 contrast ratio
     */
    INSUFFICIENT = 0,
    /**
     * Equivalent to WCAG A (3..4.5:1) level
     */
    MINIMUM = 1,
    /**
     * Equivalent to WCAG AA (4.5..7:1) level
     */
    GOOD = 2,
    /**
     * Equivalent to WCAG AAA (>= 7:1) level
     */
    HIGH = 3
}
type Contrast = {
    readonly value: number;
    readonly level: ContrastLevel;
    toString(): string;
};
type ContrastChecker = (background: Color, foreground: Color) => Contrast;
/**
 * Computes the WCAG 2.1 contrast ratio between a background color and a
 * foreground color.
 *
 * @example
 *
 * ```ts
 * const contrast = getWCAGContrast(backgroundColor, foregroundColor);
 * console.log(contrast.value); // Output the contrast value, e.g. `4.5`
 * console.log(contrast.toString()); // Output the contrast value with description, e.g. `"4.5 AA"`
 * ```
 *
 * @param background - The background color.
 * @param foreground - The foreground color.
 * @returns An object representing the contrast value and its description.
 */
declare const getWCAGContrast: ContrastChecker;
/**
 * Creates a Color instance from RGB values.
 *
 * @example
 *
 * ```ts
 * const color = rgb(255, 87, 51);
 * ```
 *
 * @param r - The red component (0-255).
 * @param g - The green component (0-255).
 * @param b - The blue component (0-255).
 * @returns A new Color instance.
 */
declare const rgb: (r: number, g: number, b: number) => Color;
/**
 * Creates a Color instance from RGBA values.
 *
 * @example
 *
 * ```ts
 * const color = rgba(255, 87, 51, 0.5);
 * ```
 *
 * @param r - The red component (0-255).
 * @param g - The green component (0-255).
 * @param b - The blue component (0-255).
 * @param a - The alpha component (0-1).
 * @returns A new Color instance.
 */
declare const rgba: (r: number, g: number, b: number, a: number) => Color;
/**
 * Creates a Color instance from HSL values.
 *
 * @example
 *
 * ```ts
 * const color = hsl(120, 100, 50);
 * ```
 *
 * @param h - The hue component (0-360).
 * @param s - The saturation component (0-100).
 * @param l - The lightness component (0-100).
 * @returns A new Color instance.
 */
declare const hsl: (h: number, s: number, l: number) => Color;
/**
 * Creates a Color instance from HSLA values.
 *
 * @example
 *
 * ```ts
 * const color = hsla(120, 100, 50, 0.5);
 * ```
 *
 * @param h - The hue component (0-360).
 * @param s - The saturation component (0-100).
 * @param l - The lightness component (0-100).
 * @param a - The alpha component (0-1).
 * @returns A new Color instance.
 */
declare const hsla: (h: number, s: number, l: number, a: number) => Color;
/**
 * Creates a Color instance from a hexadecimal color string.
 *
 * @example
 *
 * ```ts
 * const color = hex("#ff5733");
 * ```
 *
 * @param color - The hexadecimal color string (e.g., "#ff5733" or "ff5733").
 * @returns A new Color instance.
 * @throws TypeError if the color format is unknown.
 */
declare const hex: (color: string) => Color;
type Easing = (x: number) => number;
type Options = {
    lightnessMin?: number;
    lightnessMax?: number;
    easing?: Easing;
};
/**
 * A color scale that generates a sequence of shades at uniform lightness steps
 * between a minimum and a maximum, while keeping hue and saturation fixed.
 */
declare class UniformLightnessColorScale extends ColorScale {
    readonly baseColor: Color;
    readonly lightnessMin: number;
    readonly lightnessMax: number;
    readonly easing: Easing;
    readonly steps: number;
    /**
     * Creates an instance of UniformLightnessColorScale.
     *
     * @param base - The base color for the scale.
     * @param steps - The number of steps in the color scale.
     * @param options - Additional options for configuring the lightness range and
     *   easing function.
     */
    constructor(base: ColorLikeValue, steps?: number, options?: Options);
    /**
     * Gets the length of the color scale.
     *
     * @returns The number of steps in the color scale.
     */
    get length(): number;
    /**
     * Gets the shades in the color scale, computed based on the lightness values.
     *
     * @example
     *
     * ```ts
     * const shades = colorScale.shades;
     * ```
     *
     * @returns An array of Shade instances.
     */
    get shades(): Shade[];
    /**
     * Creates a new UniformLightnessColorScale with reversed lightness values.
     *
     * @example
     *
     * ```ts
     * const reversedScale = colorScale.reversed();
     * ```
     *
     * @returns A new UniformLightnessColorScale instance with reversed lightness
     *   values.
     */
    reversed(): UniformLightnessColorScale;
}
/**
 * Hook to get a color scale (palette) by its name from the current theme.
 *
 * @example
 *
 * ```ts
 * const palette = usePalette("primary");
 * ```
 *
 * @param name - The reference to the palette in the theme.
 * @returns The color scale corresponding to the given palette name.
 */
declare const usePalette: <T extends UnknownTheme = DesignSystemThemes>(name: Theme.Reference<T, "palette">) => ColorScale;
/**
 * No easing, no acceleration
 */
declare const linear: (t: number) => number;
/**
 * Accelerating from zero velocity
 */
declare const easeInQuad: (t: number) => number;
/**
 * Decelerating to zero velocity
 */
declare const easeOutQuad: (t: number) => number;
/**
 * Acceleration until halfway, then deceleration
 */
declare const easeInOutQuad: (t: number) => number;
/**
 * Accelerating from zero velocity
 */
declare const easeInCubic: (t: number) => number;
/**
 * Decelerating to zero velocity
 */
declare const easeOutCubic: (t: number) => number;
/**
 * Acceleration until halfway, then deceleration
 */
declare const easeInOutCubic: (t: number) => number;
/**
 * Accelerating from zero velocity
 */
declare const easeInQuart: (t: number) => number;
/**
 * Decelerating to zero velocity
 */
declare const easeOutQuart: (t: number) => number;
/**
 * Acceleration until halfway, then deceleration
 */
declare const easeInOutQuart: (t: number) => number;
/**
 * Accelerating from zero velocity
 */
declare const easeInQuint: (t: number) => number;
/**
 * Decelerating to zero velocity
 */
declare const easeOutQuint: (t: number) => number;
/**
 * Acceleration until halfway, then deceleration
 */
declare const easeInOutQuint: (t: number) => number;
/**
 * Hyperbolic tangent, normalized to be bounded in the interval [0, 1]
 */
declare const tanh: (t: number) => number;
export { Color, ColorLikeValue, ColorContext, ColorProvider, ColorConsumer, useCurrentForeground, useCurrentBackground, useCurrentPalette, ColorPicker, transparent, mediumVioletRed, deepPink, paleVioletRed, hotPink, lightPink, pink, darkRed, red, firebrick, crimson, indianRed, lightCoral, salmon, darkSalmon, lightSalmon, orangeRed, tomato, darkOrange, coral, orange, darkKhaki, gold, khaki, peachPuff, yellow, paleGoldenrod, moccasin, papayaWhip, lightGoldenrodYellow, lemonChiffon, lightYellow, maroon, brown, saddleBrown, sienna, chocolate, darkGoldenrod, peru, rosyBrown, goldenrod, sandyBrown, tan, burlywood, wheat, navajoWhite, bisque, blanchedAlmond, cornsilk, darkGreen, green, darkOliveGreen, forestGreen, seaGreen, olive, oliveDrab, mediumSeaGreen, limeGreen, lime, springGreen, mediumSpringGreen, darkSeaGreen, mediumAquamarine, yellowGreen, lawnGreen, chartreuse, lightGreen, greenYellow, paleGreen, teal, darkCyan, lightSeaGreen, cadetBlue, darkTurquoise, mediumTurquoise, turquoise, aqua, cyan, aquamarine, paleTurquoise, lightCyan, navy, darkBlue, mediumBlue, blue, midnightBlue, royalBlue, steelBlue, dodgerBlue, deepSkyBlue, cornflowerBlue, skyBlue, lightSkyBlue, lightSteelBlue, lightBlue, powderBlue, indigo, purple, darkMagenta, darkViolet, darkSlateBlue, blueViolet, darkOrchid, fuchsia, magenta, slateBlue, mediumSlateBlue, mediumOrchid, mediumPurple, orchid, violet, plum, thistle, lavender, mistyRose, antiqueWhite, linen, beige, whiteSmoke, lavenderBlush, oldLace, aliceBlue, seashell, ghostWhite, honeydew, floralWhite, azure, mintCream, snow, ivory, white, black, darkSlateGray, dimGray, slateGray, gray, lightSlateGray, darkGray, silver, lightGray, gainsboro, ColorScale, FixedColorScale, getWCAGContrast, ContrastLevel, ContrastChecker, HexColorString, isHexColorString, parseHexColorString, toHexColorString, Shade, rgb, rgba, hsl, hsla, hex, UniformLightnessColorScale, usePalette, linear, easeInQuad, easeOutQuad, easeInOutQuad, easeInCubic, easeOutCubic, easeInOutCubic, easeInQuart, easeOutQuart, easeInOutQuart, easeInQuint, easeOutQuint, easeInOutQuint, tanh, View, Text, Image, Input, Upload, StaticStyle, createThemeSxProcessor, resolveThemedStyleProperties, ThemedStyle as Style, ThemedStyle, isConvertibleToStyle, useStaticStyle, useThemedStyle, SolidFill, SolidBorder, RoundedCorners, DropShadow, Rectangle, SurfaceParts, Surface, SurfaceSpec, useSurface, AvailableThemes, DesignSystemThemes, ComputedTheme, AnyComputedTheme, UnknownComputedTheme, units, u, CurrentTheme, TFC, Theme, AnyTheme, UnknownTheme, ThemeConsumer, ThemeProvider, useTheme, Font, FontSpec, FontCatalog, FontDesignation, FontType, FontFamily, FontLoader, FontStyle, FontWeightMap, FontWeight, TextTransform, useTypography };
export type { Angle, AsProp, Length, StyleProperties, SxProp } from "@compositive/primitives";
export type { ThemedStyleProperties, ConvertibleToStyle, ThemedSxProp, ThemedSxPropItem };
//# sourceMappingURL=__entries__/index.d.ts.map