import { __decorate } from 'tslib';
import { memoized, memoize } from '@compositive/commons-memoize';
import { inspect } from '@compositive/commons-util/debug';
import { View as View$1, Text as Text$1, Image as Image$1, Input as Input$1, Upload as Upload$1, Style, isColorHSL, isColorRGB, SxProcessorProvider } from '@compositive/primitives';
import { HSL, sRGB, to, contrastWCAG21 } from 'colorjs.io/fn';
import { createPredicate, Equals, isDefined, areEqual, isString } from '@compositive/commons-predicates';
import { jsx, jsxs } from 'react/jsx-runtime';
import { createClientContext } from '@compositive/commons-react';
import { enumerable, mapObject, compactObject, asArray, freeze, pick, coalesce, titleize } from '@compositive/commons-util';
import { forwardRef, useMemo, useEffect, useContext, useState, useCallback } from 'react';
import { ChainEvaluator, Spec } from '@compositive/commons-spec';
import { RecordCache, LRUCache, ErrorCache, WeakKeyCache } from '@compositive/commons-memoize/caches';
import { every, Sequence } from '@compositive/commons-sequences';
import hsluv from 'hsluv';

const View = View$1;
const Text = Text$1;
const Image = Image$1;
const Input = Input$1;
const Upload = Upload$1;
class StaticStyle extends Style {
}

const RRGGBBAA = /^#([\da-fA-F]{2})([\da-fA-F]{2})([\da-fA-F]{2})([\da-fA-F][\da-fA-F])?$/i;
const RGBA = /^#([\da-fA-F])([\da-fA-F])([\da-fA-F])([\da-fA-F])?$/i;
/**
 * Predicate to check if a string is a valid hex color string.
 *
 * @example
 *
 * ```ts
 * isHexColorString("#ff5733"); // true
 * isHexColorString("#f53"); // true
 * isHexColorString("123456"); // false
 * ```
 */
const isHexColorString = createPredicate((value) => typeof value === "string" && (RRGGBBAA.test(value) || RGBA.test(value)), "a hex color representation");
const REPEATED_RRGGBBAA = /#([\da-fA-F])\1([\da-fA-F])\2([\da-fA-F])\3(([\da-fA-F])\5)?$/i;
/**
 * Parses a hex color string to a ColorRGB object.
 *
 * @example
 *
 * ```ts
 * const rgb = parseHexColorString("#ff5733");
 * // rgb: { r: 255, g: 87, b: 51, a: 1 }
 * ```
 *
 * @param color - The hex color string to parse.
 * @returns An object representing the color in RGB format.
 */
function parseHexColorString(color) {
    let match = RRGGBBAA.exec(color);
    if (match) {
        return {
            r: parseInt(match[1], 16),
            g: parseInt(match[2], 16),
            b: parseInt(match[3], 16),
            a: parseInt(match[4] ?? "ff", 16) / 255,
        };
    }
    match = RGBA.exec(color);
    if (match) {
        return {
            r: parseInt(match[1], 16) * 17,
            g: parseInt(match[2], 16) * 17,
            b: parseInt(match[3], 16) * 17,
            a: parseInt(match[4] ?? "f", 16) / 15,
        };
    }
    throw new TypeError(`Cannot parse color: '${color}'`);
}
/**
 * Converts a ColorRGB object to a hex color string.
 *
 * @example
 *
 * ```ts
 * const hex = toHexColorString({ r: 255, g: 87, b: 51, a: 1 });
 * // hex: "#ff5733"
 * ```
 *
 * @param rgb - The RGB color object to convert.
 * @returns The hex color string.
 */
function toHexColorString(rgb) {
    let color = "#" +
        [rgb.r, rgb.g, rgb.b]
            .map((channel) => channel.toString(16).padStart(2, "0"))
            .join("");
    const alpha = rgb.a ?? 1;
    if (alpha !== 1) {
        color += Math.round(alpha * 255)
            .toString(16)
            .padStart(2, "0");
    }
    const match = REPEATED_RRGGBBAA.exec(color);
    if (match) {
        const [, r, g, b, , a] = match;
        color = `#${r}${g}${b}${a ?? ""}`;
    }
    return color;
}

/**
 * Class representing a color and its various transformations.
 */
class Color {
    /**
     * Creates a Color instance from a binary representation.
     *
     * @param binary - The binary representation of the color.
     * @returns A new Color instance.
     */
    static fromBinary([r, g, b]) {
        return new Color({ r, g, b });
    }
    #colorObj;
    /**
     * Creates an instance of Color.
     *
     * @param colorValue - The value representing the color.
     */
    constructor(colorValue) {
        if (colorValue?.constructor === Color)
            return colorValue;
        if (isColorHSL(colorValue)) {
            const { h, s, l, a } = colorValue;
            this.#colorObj = { space: HSL, coords: [h, s, l], alpha: a ?? 1 };
        }
        else {
            let rgb;
            if (isHexColorString(colorValue))
                rgb = parseHexColorString(colorValue);
            else if (colorValue instanceof Color || isColorRGB(colorValue))
                rgb = colorValue;
            else
                throw new TypeError(`Unknown color value: ${inspect(colorValue)}`);
            this.#colorObj = {
                space: sRGB,
                coords: [rgb.r / 255, rgb.g / 255, rgb.b / 255],
                alpha: rgb.a ?? 1,
            };
        }
    }
    /**
     * Returns a style that uses the color as foreground.
     *
     * @returns The static style for the foreground color.
     */
    asForeground() {
        return new StaticStyle({ color: this });
    }
    /**
     * Returns a style that uses the color as background.
     *
     * @returns The static style for the background color.
     */
    asBackground() {
        return new StaticStyle({ backgroundColor: this });
    }
    /**
     * Converts the color to a static style that uses it as the foreground.
     *
     * @returns The static style for the color.
     * @see this#asForeground()
     */
    toStaticStyle() {
        return this.asForeground();
    }
    /**
     * Gets the hexadecimal representation of the color.
     *
     * @returns The hexadecimal color string.
     */
    get hex() {
        return toHexColorString(this);
    }
    /**
     * Gets the red component of the color.
     *
     * @returns The red component value.
     */
    get r() {
        return this.rgb[0];
    }
    /**
     * Gets the green component of the color.
     *
     * @returns The green component value.
     */
    get g() {
        return this.rgb[1];
    }
    /**
     * Gets the blue component of the color.
     *
     * @returns The blue component value.
     */
    get b() {
        return this.rgb[2];
    }
    get h() {
        return this.hsl[0];
    }
    get s() {
        return this.hsl[1];
    }
    get l() {
        return this.hsl[2];
    }
    /**
     * Gets the alpha (transparency) component of the color.
     *
     * @returns The alpha component value.
     */
    get a() {
        return this.#colorObj.alpha ?? 1;
    }
    /**
     * Gets the RGB representation of the color.
     *
     * @returns An array containing the RGB values.
     */
    get rgb() {
        return to(this.#colorObj, sRGB).coords.map((c) => Math.round(c * 255));
    }
    /**
     * Gets the HSL representation of the color.
     *
     * @returns An array containing the HSL values.
     */
    get hsl() {
        return to(this.#colorObj, HSL).coords;
    }
    alpha(alpha) {
        const { r, g, b } = this;
        return new Color({ r, g, b, a: alpha });
    }
    toString() {
        return this.hex;
    }
    toBinary() {
        return new Uint8Array([this.r, this.g, this.b]);
    }
}
__decorate([
    memoized
], Color.prototype, "asForeground", null);
__decorate([
    memoized
], Color.prototype, "asBackground", null);
__decorate([
    memoized
], Color.prototype, "hex", null);
__decorate([
    memoized
], Color.prototype, "rgb", null);
__decorate([
    memoized
], Color.prototype, "hsl", null);

const byVariant = {
    caches: (i) => i === 0
        ? new RecordCache(new LRUCache({ limit: 16 }))
        : ErrorCache,
};
const byTheme = {
    caches: (i) => (i === 0 ? new WeakKeyCache() : ErrorCache),
};

/**
 * Abstract class representing a theme with attributes and variants.
 */
class Theme {
    /**
     * Retrieves all fonts used in the theme. Uses memoization to cache the
     * result.
     */
    get allFonts() {
        return Array.from(Object.values(this.typography).reduce((set, spec) => spec == null
            ? set
            : spec.allFonts.reduce((acc, font) => acc.add(font), set), new Set()));
    }
    /**
     * Loads all fonts used in the theme asynchronously.
     *
     * @example
     *
     * ```typescript
     * await theme.loadAllFonts();
     * ```
     */
    async loadAllFonts() {
        await every(this.allFonts)
            .groupBy((font) => font.fontLoader)
            .mapAsync(([loader, fonts]) => loader.loadMany(fonts));
    }
    /**
     * Checks if all fonts used in the theme are loaded.
     *
     * @example
     *
     * ```typescript
     * const loaded = theme.areFontsLoaded();
     * ```
     */
    areFontsLoaded() {
        return this.allFonts.every((font) => font.isLoaded());
    }
}
__decorate([
    memoized
], Theme.prototype, "allFonts", null);

// FIXME: The previous type in the definition thunks for non-existing attributes should be "never" instead of "unknown"
// FIXME: Variant options should merge, not be replaced
/**
 * A {@link ComputedTheme} is a {@link Theme} implementation that allows for its
 * values to be derived through the use of pure functions and derivation rules.
 *
 * It's possible to create new themes from scratch, or through the extension of
 * other themes, by overriding only some of the values.
 */
class ComputedTheme extends Theme {
    /**
     * Creates a new ComputedTheme with the given definition.
     *
     * @example
     *
     * ```tsx
     * const theme = ComputedTheme.with({
     *   palette: {
     *     primary: new UniformLightnessColorScale("#0ff", 10),
     *     neutral: new UniformLightnessColorScale("#000", 10),
     *     info: new UniformLightnessColorScale("#00f", 10),
     *     warning: new UniformLightnessColorScale("#ff0", 10),
     *     danger: new UniformLightnessColorScale("#f00", 10),
     *     success: new UniformLightnessColorScale("#0f0", 10),
     *   },
     *
     *   colorPicks: {
     *     foreground: new ColorPicker((palette) => palette.frontmost),
     *     background: new ColorPicker((palette) => palette.backmost),
     *   },
     *
     *   typography: {
     *     default: HelveticaNeue.Regular.withSize(14),
     *   },
     *
     *   metrics: {
     *     grid: 4,
     *   },
     *
     *   surfaces: {
     *     default: new SurfaceSpec({ shape: new Rectangle({}) }),
     *   },
     * });
     * ```
     *
     * @param definition - The full attribute definition for the theme.
     * @returns A new ComputedTheme instance.
     */
    static with(definition) {
        return new ComputedTheme([
            { definition, variants: {} },
        ]);
    }
    #chain;
    constructor(chain) {
        super();
        this.#chain = chain;
    }
    /**
     * Returns a new Theme instance with the given active variants.
     *
     * @example
     *
     * ```tsx
     * const darkTheme = theme.using({ mode: "dark" });
     * ```
     *
     * @param activeVariants - The active variants to use.
     * @returns A new Theme instance.
     */
    using(activeVariants) {
        return new ThemeSnapshot(this.#buildEvaluator(activeVariants), this, activeVariants);
    }
    /**
     * Checks if the given object is equal to this ComputedTheme instance.
     *
     * @param other - The object to compare.
     * @returns `true` if the objects are equal, `false` otherwise.
     */
    [Equals](other) {
        return this === other;
    }
    /**
     * Creates a new ComputedTheme with additional attribute definitions.
     *
     * @example
     *
     * ```tsx
     * const newTheme = computedTheme.with({ metrics: { grid: 8 } });
     * ```
     *
     * @param definition - The additional attribute definitions.
     * @returns A new ComputedTheme instance.
     */
    with(definition) {
        return new ComputedTheme([
            ...this.#chain,
            { definition, variants: {} },
        ]);
    }
    /**
     * Creates a new ComputedTheme with additional variant options.
     *
     * @example
     *
     * ```tsx
     * const newTheme = computedTheme.when("mode", {
     *   light: {},
     *   dark: {
     *     palette: {
     *       primary: (prev) => prev.reversed(),
     *       secondary: (prev) => prev.reversed(),
     *       neutral: (prev) => prev.reversed(),
     *       info: (prev) => prev.reversed(),
     *       warning: (prev) => prev.reversed(),
     *       danger: (prev) => prev.reversed(),
     *       success: (prev) => prev.reversed(),
     *     },
     *   },
     * });
     * ```
     *
     * @param variantName - The name of the variant.
     * @param options - The variant options.
     * @returns A new ComputedTheme instance.
     */
    when(variantName, options) {
        return new ComputedTheme([
            ...this.#chain,
            { definition: {}, variants: { [variantName]: options } },
        ]);
    }
    get surfaces() {
        return this.using({}).surfaces;
    }
    get typography() {
        return this.using({}).typography;
    }
    get metrics() {
        return this.using({}).metrics;
    }
    get palette() {
        return this.using({}).palette;
    }
    get colorPicks() {
        return this.using({}).colorPicks;
    }
    get variants() {
        const options = {};
        for (const { variants } of this.#chain) {
            for (const [name, definitions] of Object.entries(variants)) {
                options[name] = [
                    ...new Set([...(options[name] ?? []), ...Object.keys(definitions)]),
                ];
            }
        }
        return options;
    }
    #buildEvaluator(activeVariants = {}) {
        const sortedActiveVariantKeys = Object.keys(activeVariants).sort();
        const definitions = [];
        for (const { definition, variants } of this.#chain) {
            definitions.push(definition);
            for (const variantName of sortedActiveVariantKeys) {
                const activeOption = activeVariants[variantName];
                if (activeOption == null)
                    continue;
                const variantDefinition = variants[variantName]?.[activeOption];
                if (variantDefinition != null)
                    definitions.push(variantDefinition);
            }
        }
        return ChainEvaluator.create(1, definitions);
    }
}
__decorate([
    memoized(byVariant)
], ComputedTheme.prototype, "using", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "surfaces", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "typography", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "metrics", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "palette", null);
__decorate([
    enumerable
], ComputedTheme.prototype, "colorPicks", null);
__decorate([
    memoized
], ComputedTheme.prototype, "variants", null);
/**
 * A snapshot of a theme at a specific point in time.
 */
class ThemeSnapshot extends Theme {
    /**
     * The evaluator for the theme attributes.
     */
    #evaluator;
    /**
     * The origin ComputedTheme instance.
     */
    #origin;
    /**
     * The active variants for the theme.
     */
    #activeVariants;
    /**
     * Constructs a new ThemeSnapshot instance.
     *
     * @param evaluator - The evaluator for the theme attributes.
     * @param origin - The origin ComputedTheme instance.
     * @param activeVariants - The active variants for the theme.
     */
    constructor(evaluator, origin, activeVariants) {
        super();
        this.#evaluator = evaluator;
        this.#origin = origin;
        this.#activeVariants = activeVariants;
    }
    /**
     * Returns a new Theme instance with the given active variants.
     *
     * @example
     *
     * ```tsx
     * const theme = themeSnapshot.using({ variantName: "option" });
     * ```
     *
     * @param activeVariants - The active variants to use.
     * @returns A new Theme instance.
     */
    using(activeVariants) {
        return this.#origin.using({ ...this.#activeVariants, ...activeVariants });
    }
    /**
     * Gets the surfaces attribute.
     *
     * @returns The surfaces attribute.
     */
    get surfaces() {
        return this.#evaluator.surfaces;
    }
    /**
     * Gets the typography attribute.
     *
     * @returns The typography attribute.
     */
    get typography() {
        return this.#evaluator.typography;
    }
    /**
     * Gets the metrics attribute.
     *
     * @returns The metrics attribute.
     */
    get metrics() {
        return this.#evaluator.metrics;
    }
    /**
     * Gets the palette attribute.
     *
     * @returns The palette attribute.
     */
    get palette() {
        return this.#evaluator.palette;
    }
    /**
     * Gets the color picks attribute.
     *
     * @returns The color picks attribute.
     */
    get colorPicks() {
        return this.#evaluator.colorPicks;
    }
    /**
     * Gets the variant options.
     *
     * @returns The variant options.
     */
    get variants() {
        return this.#origin.variants;
    }
}
__decorate([
    enumerable
], ThemeSnapshot.prototype, "surfaces", null);
__decorate([
    enumerable
], ThemeSnapshot.prototype, "typography", null);
__decorate([
    enumerable
], ThemeSnapshot.prototype, "metrics", null);
__decorate([
    enumerable
], ThemeSnapshot.prototype, "palette", null);
__decorate([
    enumerable
], ThemeSnapshot.prototype, "colorPicks", null);

/**
 * Generates a length value based on the theme's grid metric and a multiplier.
 *
 * @example
 *
 * ```tsx
 * // If the currently active theme has a `metrics.grid` value of 8px, the padding for the component will be 16px
 * const padding = new Style({ padding: units(2) });
 * ```
 *
 * @param multiplier - A number to multiply the theme's grid metric by. Defaults
 *   to 1.
 * @returns A function that takes a theme and returns a length value as a
 *   string.
 */
const units = (multiplier = 1) => (theme) => `${theme.metrics.grid * multiplier}px`;
/**
 * Shorthand for the {@linkcode units()} function.
 */
const u = units;

/**
 * Utility for creating a themed functional component with a ref.
 *
 * @example
 *
 * ```tsx
 * const ThemedComponent = TFC.forwardRef((props, ref) => {
 *   return <View ref={ref} sx={props.sx}>Hello, themed world!</div>;
 * });
 * ```
 */
const TFC = {
    forwardRef: (render) => {
        return forwardRef(render);
    },
};

/**
 * A context to provide theme information throughout the component tree.
 */
const ThemeContext = createClientContext(undefined);
/**
 * Error message to be used when no theme is found in the component hierarchy.
 * This encourages users to wrap their components in a `ThemeProvider`.
 */
const THEME_NOT_PROVIDED_ERROR = "No theme found in the component hierarchy. Please wrap your components in a ThemeProvider element.";

/**
 * A component that consumes the theme context and passes the theme value to its
 * children.
 *
 * @example
 *
 * ```tsx
 * <ThemeConsumer>
 *   {(theme) => (
 *     <div style={{ color: theme.palette.primary.darkest }}>Hello</div>
 *   )}
 * </ThemeConsumer>;
 * ```
 */
const ThemeConsumer = ({ children, }) => {
    return (jsx(ThemeContext.Consumer, { children: (theme) => children(isDefined.validate(theme, THEME_NOT_PROVIDED_ERROR)) }));
};

/**
 * Resolves themed style properties into static style properties.
 *
 * This function takes themed style properties and a theme, and resolves them
 * into static style properties by applying the theme to any functions within
 * the style properties.
 *
 * @param themedStyleProperties - The themed style properties or generator
 *   function.
 * @param theme - The theme to use for resolving style properties.
 * @returns The resolved static style properties.
 * @internal
 */
const resolveThemedStyleProperties = memoize((themedStyleProperties, theme) => {
    const styleProps = typeof themedStyleProperties === "function"
        ? Reflect.apply(themedStyleProperties, null, [theme])
        : themedStyleProperties;
    return mapObject(styleProps, (value) => (typeof value === "function"
        ? Reflect.apply(value, null, [theme])
        : value));
}, { argCaches: (i) => (i < 2 ? new WeakKeyCache() : ErrorCache) });

/**
 * Checks if a value is convertible to a StaticStyle.
 *
 * @example
 *
 * ```typescript
 * const value = { toStaticStyle: (theme) => new StaticStyle() };
 * const result = isConvertibleToStyle(value); // true
 * ```
 *
 * @param value - The value to check.
 * @returns True if the value is convertible to a StaticStyle, otherwise false.
 */
const isConvertibleToStyle = (value) => {
    return (typeof value === "object" &&
        value != null &&
        typeof value.toStaticStyle === "function");
};

/**
 * Creates a theme-specific style processor function.
 *
 * This function takes a theme and returns another function that processes
 * themed style properties into either a StaticStyle or StyleProperties.
 *
 * @example
 *
 * ```typescript
 * const theme = { colors: { primary: "blue" } };
 * const processor = createThemeSxProcessor(theme);
 * const style = processor({ color: "primary" });
 * ```
 *
 * @param theme - The theme to use for resolving style properties.
 * @returns A function that processes themed style properties.
 */
/* #__NO_SIDE_EFFECTS__ */
const createThemeSxProcessor = (theme) => (style) => {
    if (isConvertibleToStyle(style))
        return style.toStaticStyle(theme);
    if (style instanceof StaticStyle)
        return style;
    return resolveThemedStyleProperties(style, theme);
};

class ThemedStyle {
    // TODO: Copying this static method here is not the best API design decision.
    // It would be better to find a better structure to separate "Style" manipulation from
    // the "SxProp" manipulation
    static concat(...sxProps) {
        return StaticStyle.concat(...sxProps);
    }
    #themedStyleProperties;
    constructor(themedStyleProperties) {
        this.#themedStyleProperties = themedStyleProperties;
    }
    toStaticStyle(theme) {
        return new StaticStyle(resolveThemedStyleProperties(this.#themedStyleProperties, theme));
    }
}
__decorate([
    memoized(byTheme)
], ThemedStyle.prototype, "toStaticStyle", null);

/**
 * Hook that creates a {@linkcode StaticStyle} from style properties.
 *
 * This hook takes a function that returns style properties and a dependency
 * list. The resulting style is memoized, and recreated whenever the
 * dependencies change.
 *
 * @example
 *
 * ```typescript
 * const style = useStaticStyle(
 *   () => ({ color: foregroundColor }),
 *   [foregroundColor],
 * );
 * ```
 *
 * @param rules - A function that returns style properties.
 * @param deps - A list of dependencies that, when changed, will cause the
 *   StaticStyle to be re-created.
 * @returns A StaticStyle instance.
 */
const useStaticStyle = (rules, deps) => useMemo(() => new StaticStyle(rules()), 
// eslint-disable-next-line react-hooks/exhaustive-deps
deps);

/**
 * A hook that creates a ThemedStyle from themed style properties.
 *
 * This hook takes a function that returns themed style properties and a
 * dependency list. The resulting style is cached, and recreated only when one
 * of the dependencies change.
 *
 * @example
 *
 * ```typescript
 * const themedStyle = useThemedStyle(
 *   () => ({ width: (theme) => theme.grid.unit * size }),
 *   [size],
 * );
 * ```
 *
 * @param rules - A function that returns themed style properties.
 * @param deps - A list of dependencies that, when changed, will cause the
 *   ThemedStyle to be re-created.
 * @returns A ThemedStyle instance.
 */
const useThemedStyle = (rules, deps) => useMemo(() => new ThemedStyle(rules()), 
// eslint-disable-next-line react-hooks/exhaustive-deps
deps);

/**
 * A provider component that supplies a theme to its children. It's possible to
 * specify which theme variants should be evaluated as active in the current
 * tree.
 *
 * @example
 *
 * ```tsx
 * <ThemeProvider theme={Rubik} mode="dark">
 *   <MyComponent />
 * </ThemeProvider>;
 * ```
 */
const ThemeProvider = ({ theme, children, ...variants }) => {
    // Memoize the theme with variants to avoid unnecessary recalculations
    const themeWithVariants = useMemo(() => (Object.keys(variants).length === 0 ? theme : theme.using(variants)), [theme, variants]);
    // Load all fonts for the theme when the themeWithVariants changes
    useEffect(() => {
        themeWithVariants
            .loadAllFonts()
            .catch((error) => console.error("Error while trying to load the theme fonts:", error));
    }, [themeWithVariants]);
    // Memoize the theme Sx processor to avoid unnecessary recalculations
    const themeSxProcessor = useMemo(() => createThemeSxProcessor(themeWithVariants), [themeWithVariants]);
    return (jsx(ThemeContext.Provider, { value: themeWithVariants, children: jsx(SxProcessorProvider, { value: themeSxProcessor, children: children }) }));
};

/**
 * A custom hook that retrieves the current theme from the ThemeContext. It
 * asserts that the theme is defined and throws an error if it is not.
 *
 * @example
 *
 * ```tsx
 * const theme = useTheme();
 * console.log(theme);
 * ```
 */
const useTheme = () => {
    const theme = useContext(ThemeContext);
    isDefined.assert(theme, THEME_NOT_PROVIDED_ERROR);
    return theme;
};

/**
 * Class representing a color picker that selects a shade from a color scale.
 */
class ColorPicker {
    #colorPickingFn;
    /**
     * Creates an instance of ColorPicker.
     *
     * @param colorPickingFn - The function used to pick a color from the color
     *   scale.
     */
    constructor(colorPickingFn) {
        this.#colorPickingFn = colorPickingFn;
    }
    /**
     * Picks a shade from the given color scale, optionally considering a
     * background shade.
     *
     * @example
     *
     * ```ts
     * const pickedShade = colorPicker.pickColor(colorScale, backgroundShade);
     * ```
     *
     * @param palette - The color scale to pick a shade from.
     * @param background - An optional background shade that might influence the
     *   picked color.
     * @returns The picked shade.
     */
    pickColor(palette, background) {
        return this.#colorPickingFn(palette, background);
    }
}

/**
 * Representation of a color part of a certain color scale
 */
class Shade extends Color {
    colorScale;
    index;
    /**
     * Creates an instance of Shade.
     *
     * @param colorScale - The color scale to which this shade belongs.
     * @param index - The index of the shade in the color scale.
     * @param colorValue - The color value of the shade.
     */
    constructor(colorScale, index, colorValue) {
        super(colorValue);
        this.colorScale = colorScale;
        this.index = index;
    }
    /**
     * Transposes this shade onto another color scale at the same index.
     *
     * @example
     *
     * ```ts
     * const transposedShade = shade.transposeOn(otherScale);
     * ```
     *
     * @param otherScale - The color scale to transpose onto.
     * @returns The shade from the other scale at the same index, or undefined if
     *   out of bounds.
     */
    transposeOn(otherScale) {
        return otherScale.get(this.index);
    }
    backward(steps = 1, outOfBoundariesMethod = "none") {
        return this.colorScale.get(this.index - steps, outOfBoundariesMethod);
    }
    frontward(steps = 1, outOfBoundariesMethod = "none") {
        return this.colorScale.get(this.index + steps, outOfBoundariesMethod);
    }
    /**
     * Checks if another object is equal to this shade.
     *
     * @param other - The object to compare with.
     * @returns True if the objects are equal, otherwise false.
     */
    [Equals](other) {
        const ShadeClass = this.constructor;
        return (other instanceof ShadeClass &&
            other.hex === this.hex &&
            other.index === this.index);
    }
}

const DEFAULT_PROPS = {
    foreground: "foreground",
    background: "background",
    palette: "neutral",
};
const ColorContext = createClientContext(DEFAULT_PROPS);
/**
 * A provider component that supplies color context values to its children. The
 * values passed can be references to the `colorPicks` section of the theme,
 * {@link Shade|shades} or {@link ColorPicker|color-picking objects}.
 *
 * In all cases the values will be resolved to color shades before being passed
 * to the child components.
 *
 * @example
 *
 * ```tsx
 * <ColorProvider
 *   foreground={theme.palette.neutral.frontmost}
 *   background="background"
 *   palette="neutral"
 * >
 *   <YourComponent />
 * </ColorProvider>;
 * ```
 */
const ColorProvider = (props) => {
    const previousInput = useContext(ColorContext);
    const { foreground, background, palette, children } = {
        ...previousInput,
        ...compactObject(props),
    };
    return (jsx(ColorContext.Provider, { value: { foreground, background, palette }, children: children }));
};
/**
 * A consumer component that receives the current color context values and
 * passes them to its children function.
 *
 * @example
 *
 * ```tsx
 * <ColorConsumer>
 *   {({ foreground, background, palette }) => (
 *     <div style={{ color: foreground }}>
 *       Background color is {background.hex} and the palette has{" "}
 *       {palette.length} shades.
 *     </div>
 *   )}
 * </ColorConsumer>;
 * ```
 */
const ColorConsumer = ({ children, }) => {
    const values = useCurrentColors();
    return children(values);
};
/**
 * Resolves the appropriate shade based on the provided color pick.
 *
 * @example
 *
 * ```tsx
 * const shade = resolveShade(theme, palette, colorPick);
 * ```
 *
 * @param theme - The theme object containing color picks.
 * @param palette - The color scale palette to use.
 * @param pick - The color pick to resolve.
 * @param background - Optional background shade for context.
 * @returns The resolved shade.
 */
const resolveShade = (theme, palette, pick, background) => {
    if (pick instanceof Shade) {
        return pick;
    }
    if (pick instanceof ColorPicker) {
        return pick.pickColor(palette, background);
    }
    const validPickName = isDefined.validate(asArray(pick).find((name) => theme.colorPicks[name] != null));
    return resolveShade(theme, palette, theme.colorPicks[validPickName], background);
};
/**
 * Hook to get the current color context values.
 *
 * @example
 *
 * ```tsx
 * const { foreground, background, palette } = useCurrentColors();
 * ```
 *
 * @returns The current color context output including foreground, background,
 *   and palette shades.
 */
const useCurrentColors = () => {
    const refs = useContext(ColorContext);
    const theme = useTheme();
    const palette = theme.palette[refs.palette];
    const background = resolveShade(theme, palette, refs.background);
    const foreground = resolveShade(theme, palette, refs.foreground, background);
    return { palette, background, foreground };
};
const useCurrentForeground = () => useCurrentColors().foreground;
const useCurrentBackground = () => useCurrentColors().background;
const useCurrentPalette = () => useCurrentColors().palette;

/**
 * Creates a Color instance from RGB values.
 *
 * @example
 *
 * ```ts
 * const color = rgb(255, 87, 51);
 * ```
 *
 * @param r - The red component (0-255).
 * @param g - The green component (0-255).
 * @param b - The blue component (0-255).
 * @returns A new Color instance.
 */
const rgb = (r, g, b) => new Color({ r, g, b });
/**
 * Creates a Color instance from RGBA values.
 *
 * @example
 *
 * ```ts
 * const color = rgba(255, 87, 51, 0.5);
 * ```
 *
 * @param r - The red component (0-255).
 * @param g - The green component (0-255).
 * @param b - The blue component (0-255).
 * @param a - The alpha component (0-1).
 * @returns A new Color instance.
 */
const rgba = (r, g, b, a) => new Color({ r, g, b, a });
/**
 * Creates a Color instance from HSL values.
 *
 * @example
 *
 * ```ts
 * const color = hsl(120, 100, 50);
 * ```
 *
 * @param h - The hue component (0-360).
 * @param s - The saturation component (0-100).
 * @param l - The lightness component (0-100).
 * @returns A new Color instance.
 */
const hsl = (h, s, l) => new Color({ h, s, l });
/**
 * Creates a Color instance from HSLA values.
 *
 * @example
 *
 * ```ts
 * const color = hsla(120, 100, 50, 0.5);
 * ```
 *
 * @param h - The hue component (0-360).
 * @param s - The saturation component (0-100).
 * @param l - The lightness component (0-100).
 * @param a - The alpha component (0-1).
 * @returns A new Color instance.
 */
const hsla = (h, s, l, a) => new Color({ h, s, l, a });
/**
 * Creates a Color instance from a hexadecimal color string.
 *
 * @example
 *
 * ```ts
 * const color = hex("#ff5733");
 * ```
 *
 * @param color - The hexadecimal color string (e.g., "#ff5733" or "ff5733").
 * @returns A new Color instance.
 * @throws TypeError if the color format is unknown.
 */
const hex = (color) => {
    let hexValue = color;
    if (hexValue.startsWith("#"))
        hexValue = hexValue.slice(1);
    if (hexValue.length === 3)
        return new Color({
            r: parseInt(hexValue.charAt(0), 16) * 0x11,
            g: parseInt(hexValue.charAt(1), 16) * 0x11,
            b: parseInt(hexValue.charAt(2), 16) * 0x11,
        });
    if (hexValue.length === 6)
        return new Color({
            r: parseInt(hexValue.slice(0, 2), 16),
            g: parseInt(hexValue.slice(2, 4), 16),
            b: parseInt(hexValue.slice(4, 6), 16),
        });
    throw new TypeError(`Unknown color format: ${color}`);
};

const transparent = rgba(0, 0, 0, 0);
// Red colors
const mediumVioletRed = rgb(199, 21, 133);
const deepPink = rgb(255, 20, 147);
const paleVioletRed = rgb(219, 112, 147);
const hotPink = rgb(255, 105, 180);
const lightPink = rgb(255, 182, 193);
const pink = rgb(255, 192, 203);
const darkRed = rgb(139, 0, 0);
const red = rgb(255, 0, 0);
const firebrick = rgb(178, 34, 34);
const crimson = rgb(220, 20, 60);
const indianRed = rgb(205, 92, 92);
const lightCoral = rgb(240, 128, 128);
const salmon = rgb(250, 128, 114);
const darkSalmon = rgb(233, 150, 122);
const lightSalmon = rgb(255, 160, 122);
// Orange colors
const orangeRed = rgb(255, 69, 0);
const tomato = rgb(255, 99, 71);
const darkOrange = rgb(255, 140, 0);
const coral = rgb(255, 127, 80);
const orange = rgb(255, 165, 0);
// Yellow colors
const darkKhaki = rgb(189, 183, 107);
const gold = rgb(255, 215, 0);
const khaki = rgb(240, 230, 140);
const peachPuff = rgb(255, 218, 185);
const yellow = rgb(255, 255, 0);
const paleGoldenrod = rgb(238, 232, 170);
const moccasin = rgb(255, 228, 181);
const papayaWhip = rgb(255, 239, 213);
const lightGoldenrodYellow = rgb(250, 250, 210);
const lemonChiffon = rgb(255, 250, 205);
const lightYellow = rgb(255, 255, 224);
// Brown colors
const maroon = rgb(128, 0, 0);
const brown = rgb(165, 42, 42);
const saddleBrown = rgb(139, 69, 19);
const sienna = rgb(160, 82, 45);
const chocolate = rgb(210, 105, 30);
const darkGoldenrod = rgb(184, 134, 11);
const peru = rgb(205, 133, 63);
const rosyBrown = rgb(188, 143, 143);
const goldenrod = rgb(218, 165, 32);
const sandyBrown = rgb(244, 164, 96);
const tan = rgb(210, 180, 140);
const burlywood = rgb(222, 184, 135);
const wheat = rgb(245, 222, 179);
const navajoWhite = rgb(255, 222, 173);
const bisque = rgb(255, 228, 196);
const blanchedAlmond = rgb(255, 235, 205);
const cornsilk = rgb(255, 248, 220);
// Green colors
const darkGreen = rgb(0, 100, 0);
const green = rgb(0, 128, 0);
const darkOliveGreen = rgb(85, 107, 47);
const forestGreen = rgb(34, 139, 34);
const seaGreen = rgb(46, 139, 87);
const olive = rgb(128, 128, 0);
const oliveDrab = rgb(107, 142, 35);
const mediumSeaGreen = rgb(60, 179, 113);
const limeGreen = rgb(50, 205, 50);
const lime = rgb(0, 255, 0);
const springGreen = rgb(0, 255, 127);
const mediumSpringGreen = rgb(0, 250, 154);
const darkSeaGreen = rgb(143, 188, 143);
const mediumAquamarine = rgb(102, 205, 170);
const yellowGreen = rgb(154, 205, 50);
const lawnGreen = rgb(124, 252, 0);
const chartreuse = rgb(127, 255, 0);
const lightGreen = rgb(144, 238, 144);
const greenYellow = rgb(173, 255, 47);
const paleGreen = rgb(152, 251, 152);
// Cyan colors
const teal = rgb(0, 128, 128);
const darkCyan = rgb(0, 139, 139);
const lightSeaGreen = rgb(32, 178, 170);
const cadetBlue = rgb(95, 158, 160);
const darkTurquoise = rgb(0, 206, 209);
const mediumTurquoise = rgb(72, 209, 204);
const turquoise = rgb(64, 224, 208);
const aqua = rgb(0, 255, 255);
const cyan = rgb(0, 255, 255);
const aquamarine = rgb(127, 255, 212);
const paleTurquoise = rgb(175, 238, 238);
const lightCyan = rgb(224, 255, 255);
// Blue colors
const navy = rgb(0, 0, 128);
const darkBlue = rgb(0, 0, 139);
const mediumBlue = rgb(0, 0, 205);
const blue = rgb(0, 0, 255);
const midnightBlue = rgb(25, 25, 112);
const royalBlue = rgb(65, 105, 225);
const steelBlue = rgb(70, 130, 180);
const dodgerBlue = rgb(30, 144, 255);
const deepSkyBlue = rgb(0, 191, 255);
const cornflowerBlue = rgb(100, 149, 237);
const skyBlue = rgb(135, 206, 235);
const lightSkyBlue = rgb(135, 206, 250);
const lightSteelBlue = rgb(176, 196, 222);
const lightBlue = rgb(173, 216, 230);
const powderBlue = rgb(176, 224, 230);
// Purple, violet, and magenta colors
const indigo = rgb(75, 0, 130);
const purple = rgb(128, 0, 128);
const darkMagenta = rgb(139, 0, 139);
const darkViolet = rgb(148, 0, 211);
const darkSlateBlue = rgb(72, 61, 139);
const blueViolet = rgb(138, 43, 226);
const darkOrchid = rgb(153, 50, 204);
const fuchsia = rgb(255, 0, 255);
const magenta = rgb(255, 0, 255);
const slateBlue = rgb(106, 90, 205);
const mediumSlateBlue = rgb(123, 104, 238);
const mediumOrchid = rgb(186, 85, 211);
const mediumPurple = rgb(147, 112, 219);
const orchid = rgb(218, 112, 214);
const violet = rgb(238, 130, 238);
const plum = rgb(221, 160, 221);
const thistle = rgb(216, 191, 216);
const lavender = rgb(230, 230, 250);
// White colors
const mistyRose = rgb(255, 228, 225);
const antiqueWhite = rgb(250, 235, 215);
const linen = rgb(250, 240, 230);
const beige = rgb(245, 245, 220);
const whiteSmoke = rgb(245, 245, 245);
const lavenderBlush = rgb(255, 240, 245);
const oldLace = rgb(253, 245, 230);
const aliceBlue = rgb(240, 248, 255);
const seashell = rgb(255, 245, 238);
const ghostWhite = rgb(248, 248, 255);
const honeydew = rgb(240, 255, 240);
const floralWhite = rgb(255, 250, 240);
const azure = rgb(240, 255, 255);
const mintCream = rgb(245, 255, 250);
const snow = rgb(255, 250, 250);
const ivory = rgb(255, 255, 240);
const white = rgb(255, 255, 255);
// Gray and black colors
const black = rgb(0, 0, 0);
const darkSlateGray = rgb(47, 79, 79);
const dimGray = rgb(105, 105, 105);
const slateGray = rgb(112, 128, 144);
const gray = rgb(128, 128, 128);
const lightSlateGray = rgb(119, 136, 153);
const darkGray = rgb(169, 169, 169);
const silver = rgb(192, 192, 192);
const lightGray = rgb(211, 211, 211);
const gainsboro = rgb(220, 220, 220);

/**
 * Abstract base class representing a color scale.
 */
class ColorScale extends Sequence {
    /**
     * Constructs a new instance of the ColorScale class.
     */
    constructor() {
        super(() => this.shades[Symbol.iterator]());
    }
    get(key, outOfBoundsMethod = "none") {
        switch (outOfBoundsMethod) {
            case "wrap": {
                if (key >= 0 && key < this.length)
                    return this.shades[key];
                const lastIndex = this.length - 1;
                const absKey = Math.abs(key);
                const rounds = Math.floor(absKey / this.length);
                const offset = absKey % lastIndex;
                return this.shades[(rounds % 2 === 0 ? lastIndex : 0) - offset * (-1) ** rounds];
            }
            case "clamp": {
                return this.shades[Math.min(this.length - 1, Math.max(0, key))];
            }
            default: {
                return this.shades[key];
            }
        }
    }
    /**
     * Checks if the shade at the specified index exists.
     *
     * @example
     *
     * ```ts
     * colorScale.has(1); // true or false
     * ```
     *
     * @param key - The index to check.
     * @returns `true` if the shade exists, otherwise `false`.
     */
    has(key) {
        return key >= 0 && key < this.length;
    }
    /**
     * Gets the number of shades in the color scale.
     *
     * @returns The length of the color scale.
     */
    get length() {
        return this.shades.length;
    }
    /**
     * Gets the backmost (first) shade in the color scale.
     *
     * @returns The backmost shade.
     */
    get backmost() {
        return this.shades[0];
    }
    /**
     * Gets the midpoint shade in the color scale.
     *
     * @returns The midpoint shade.
     */
    get midpoint() {
        const midIndex = Math.min(Math.round(this.shades.length / 2), this.shades.length - 1);
        return this.shades[midIndex];
    }
    /**
     * Gets the frontmost (last) shade in the color scale.
     *
     * @returns The frontmost shade.
     */
    get frontmost() {
        return this.shades[this.shades.length - 1];
    }
    /**
     * Filters the shades in the color scale based on a callback function.
     *
     * @example
     *
     * ```ts
     * colorScale.filter((shade) => shade.value > 50);
     * ```
     *
     * @param cb - The callback function to determine if a shade should be
     *   included.
     * @returns A new ColorScale with the filtered shades.
     */
    filter(cb) {
        return new FixedColorScale(super.filter(cb));
    }
    /**
     * Compares the current instance with another object for equality.
     *
     * @param other The object to compare with.
     * @returns True if the objects are equal, false otherwise.
     */
    [Equals](other) {
        const ColorScaleClass = this.constructor;
        return (other instanceof ColorScaleClass &&
            other.constructor === ColorScaleClass &&
            other.length === this.length &&
            areEqual([...other].map((t) => t.hex), [...this].map((t) => t.hex)));
    }
    /**
     * Builds the next color scale with the same element kind.
     *
     * @param source - The source iterable or iterator of shades.
     * @returns A new ColorScale.
     * @internal
     */
    buildNextWithSameElementKind(source) {
        return new FixedColorScale(source);
    }
}
/**
 * A color scale with a statically defined set of shades.
 */
class FixedColorScale extends ColorScale {
    /**
     * An array of `Shade` objects representing the shades in the color scale.
     */
    shades;
    /**
     * Constructs a new `FixedColorScale` instance.
     *
     * @param colors - An iterable of color values or a function that returns an
     *   iterator of color values.
     */
    constructor(colors) {
        super();
        this.shades = every(colors)
            .toArray()
            .map((c, i) => new Shade(this, i, c));
    }
    /**
     * Gets the reversed color scale.
     *
     * @example
     *
     * ```ts
     * const reversedScale = colorScale.reversed();
     * ```
     *
     * @returns A new FixedColorScale with the shades in reverse order.
     */
    reversed() {
        return new FixedColorScale([...this.shades].reverse());
    }
}

var ContrastLevel;
(function (ContrastLevel) {
    /**
     * Less than WCAG 3:1 contrast ratio
     */
    ContrastLevel[ContrastLevel["INSUFFICIENT"] = 0] = "INSUFFICIENT";
    /**
     * Equivalent to WCAG A (3..4.5:1) level
     */
    ContrastLevel[ContrastLevel["MINIMUM"] = 1] = "MINIMUM";
    /**
     * Equivalent to WCAG AA (4.5..7:1) level
     */
    ContrastLevel[ContrastLevel["GOOD"] = 2] = "GOOD";
    /**
     * Equivalent to WCAG AAA (>= 7:1) level
     */
    ContrastLevel[ContrastLevel["HIGH"] = 3] = "HIGH";
})(ContrastLevel || (ContrastLevel = {}));

/**
 * Computes the WCAG 2.1 contrast ratio between a background color and a
 * foreground color.
 *
 * @example
 *
 * ```ts
 * const contrast = getWCAGContrast(backgroundColor, foregroundColor);
 * console.log(contrast.value); // Output the contrast value, e.g. `4.5`
 * console.log(contrast.toString()); // Output the contrast value with description, e.g. `"4.5 AA"`
 * ```
 *
 * @param background - The background color.
 * @param foreground - The foreground color.
 * @returns An object representing the contrast value and its description.
 */
const getWCAGContrast = (background, foreground) => {
    const value = contrastWCAG21({
        space: sRGB,
        coords: background.rgb.map((c) => c / 255),
    }, {
        space: sRGB,
        coords: foreground.rgb.map((c) => c / 255),
    });
    return {
        /**
         * The computed contrast value.
         */
        get value() {
            return value;
        },
        /**
         * Converts the contrast value to a string, including its level description.
         *
         * @returns A string representation of the contrast value with its level
         *   description.
         */
        toString() {
            const roundedValue = Math.round(value * 100) / 100;
            const level = this.level;
            let levelDescription;
            switch (level) {
                case ContrastLevel.HIGH: {
                    levelDescription = " AAA";
                    break;
                }
                case ContrastLevel.GOOD: {
                    levelDescription = " AA";
                    break;
                }
                case ContrastLevel.MINIMUM: {
                    levelDescription = " A";
                    break;
                }
                default: {
                    levelDescription = "";
                }
            }
            return `${roundedValue}${levelDescription}`;
        },
        /**
         * Gets the contrast level based on the computed value.
         *
         * @returns The contrast level as a ContrastLevel enum value.
         */
        get level() {
            if (value >= 7)
                return ContrastLevel.HIGH;
            if (value >= 4.5)
                return ContrastLevel.GOOD;
            if (value >= 3)
                return ContrastLevel.MINIMUM;
            return ContrastLevel.INSUFFICIENT;
        },
    };
};

/**
 * No easing, no acceleration
 */
const linear = (t) => t;
/**
 * Accelerating from zero velocity
 */
const easeInQuad = (t) => t * t;
/**
 * Decelerating to zero velocity
 */
const easeOutQuad = (t) => t * (2 - t);
/**
 * Acceleration until halfway, then deceleration
 */
const easeInOutQuad = (t) => t < 0.5 ? 2 * t * t : -1 + (4 - 2 * t) * t;
/**
 * Accelerating from zero velocity
 */
const easeInCubic = (t) => t ** 3;
/**
 * Decelerating to zero velocity
 */
const easeOutCubic = (t) => (t - 1) ** 3 + 1;
/**
 * Acceleration until halfway, then deceleration
 */
const easeInOutCubic = (t) => t < 0.5 ? 4 * t ** 3 : (t - 1) * (2 * t - 2) * (2 * t - 2) + 1;
/**
 * Accelerating from zero velocity
 */
const easeInQuart = (t) => t ** 4;
/**
 * Decelerating to zero velocity
 */
const easeOutQuart = (t) => 1 - (t - 1) ** 4;
/**
 * Acceleration until halfway, then deceleration
 */
const easeInOutQuart = (t) => t < 0.5 ? 8 * t ** 4 : 1 - 8 * (t - 1) ** 4;
/**
 * Accelerating from zero velocity
 */
const easeInQuint = (t) => t ** 5;
/**
 * Decelerating to zero velocity
 */
const easeOutQuint = (t) => 1 + (t - 1) ** 5;
/**
 * Acceleration until halfway, then deceleration
 */
const easeInOutQuint = (t) => t < 0.5 ? 16 * t ** 5 : 1 + 16 * (t - 1) ** 5;
/**
 * Hyperbolic tangent, normalized to be bounded in the interval [0, 1]
 */
const tanh = (t) => Math.tanh(t * 8 - 4) / 2 + 0.5;

const { hexToHsluv, hsluvToHex } = hsluv;
/**
 * A color scale that generates a sequence of shades at uniform lightness steps
 * between a minimum and a maximum, while keeping hue and saturation fixed.
 */
class UniformLightnessColorScale extends ColorScale {
    baseColor;
    lightnessMin;
    lightnessMax;
    easing;
    steps;
    /**
     * Creates an instance of UniformLightnessColorScale.
     *
     * @param base - The base color for the scale.
     * @param steps - The number of steps in the color scale.
     * @param options - Additional options for configuring the lightness range and
     *   easing function.
     */
    constructor(base, steps = 10, options = {}) {
        super();
        this.steps = steps;
        this.baseColor = new Color(base);
        this.lightnessMin = options?.lightnessMin ?? 1;
        this.lightnessMax = options?.lightnessMax ?? 99;
        this.easing = options?.easing ?? linear;
    }
    /**
     * Gets the length of the color scale.
     *
     * @returns The number of steps in the color scale.
     */
    get length() {
        return this.steps;
    }
    /**
     * Gets the shades in the color scale, computed based on the lightness values.
     *
     * @example
     *
     * ```ts
     * const shades = colorScale.shades;
     * ```
     *
     * @returns An array of Shade instances.
     */
    get shades() {
        const [hue, saturation] = hexToHsluv(this.baseColor.hex);
        const step = 1 / (this.steps - 1);
        const lightnesses = [];
        for (let i = 0; i < this.steps; ++i)
            lightnesses[i] =
                this.lightnessMax +
                    (this.lightnessMin - this.lightnessMax) * this.easing(i * step);
        return lightnesses.map((lightness, i) => new Shade(this, i, hsluvToHex([hue, saturation, lightness])));
    }
    /**
     * Creates a new UniformLightnessColorScale with reversed lightness values.
     *
     * @example
     *
     * ```ts
     * const reversedScale = colorScale.reversed();
     * ```
     *
     * @returns A new UniformLightnessColorScale instance with reversed lightness
     *   values.
     */
    reversed() {
        return new UniformLightnessColorScale(this.baseColor, this.steps, {
            lightnessMax: this.lightnessMin,
            lightnessMin: this.lightnessMax,
            easing: this.easing,
        });
    }
}
__decorate([
    memoized
], UniformLightnessColorScale.prototype, "shades", null);

/**
 * Hook to get a color scale (palette) by its name from the current theme.
 *
 * @example
 *
 * ```ts
 * const palette = usePalette("primary");
 * ```
 *
 * @param name - The reference to the palette in the theme.
 * @returns The color scale corresponding to the given palette name.
 */
const usePalette = (name) => {
    const theme = useTheme();
    return theme.palette[name];
};

/**
 * A class representing a solid fill specification. Extends the Spec class with
 * a property for shade, and implements Surface.Element with type "background".
 *
 * @example
 *
 * ```typescript
 * const solidFill = new SolidFill({
 *   shade: theme.palette.primary.frontmost,
 * });
 * ```
 */
class SolidFill extends Spec(["shade"]) {
    type = "background";
    /**
     * Processes the surface parts and chain to apply the solid fill style.
     *
     * @param parts - The surface parts to which the styles will be added.
     * @param chain - The chain of surface elements.
     * @returns A tuple containing the updated surface parts and chain.
     */
    process(parts, chain) {
        return [parts.addContainerStyles(this._getStyle()), chain];
    }
    /**
     * Generates the static style for the solid fill.
     *
     * @returns A StaticStyle instance with the background color.
     */
    _getStyle() {
        return new StaticStyle({ backgroundColor: this.shade });
    }
}
__decorate([
    memoized
], SolidFill.prototype, "_getStyle", null);

/**
 * A class representing a solid border specification. Extends the Spec class
 * with properties for width and shade, and implements Surface.Element with type
 * "border".
 *
 * @example
 *
 * ```typescript
 * const solidBorder = new SolidBorder({
 *   width: "2px",
 *   shade: theme.palette.neutral.backmost,
 * });
 * ```
 */
class SolidBorder extends Spec(["width", "shade"]) {
    type = "border";
    /**
     * Processes the surface parts and chain to apply the solid border style.
     *
     * @param parts - The surface parts to which the styles will be added.
     * @param chain - The chain of surface elements.
     * @returns A tuple containing the updated surface parts and chain.
     */
    process(parts, chain) {
        return [parts.addContainerStyles(this._getStyle()), chain];
    }
    /**
     * Generates the static style for the solid border.
     *
     * @returns A StaticStyle instance with the border width and color.
     */
    _getStyle() {
        return new StaticStyle({
            borderWidth: this.width ?? "1px",
            borderColor: this.shade,
        });
    }
}
__decorate([
    memoized
], SolidBorder.prototype, "_getStyle", null);

/**
 * A class representing rounded corners specification. Extends the Spec class
 * with RoundedCornersAttributes and implements Surface.Element with type
 * "corners".
 *
 * @example
 *
 * ```typescript
 * const roundedCorners = new RoundedCorners({ radius: 10 });
 * ```
 */
class RoundedCorners extends Spec([
    "radius",
    "topLeft",
    "topRight",
    "bottomLeft",
    "bottomRight",
]) {
    type = "corners";
    /**
     * Constructs a new RoundedCorners instance.
     *
     * @param radiusOrAttribs - Either a number representing the radius for all
     *   corners or an object with specific corner radii.
     * @param previousSpec - An optional previous RoundedCorners specification to
     *   extend.
     */
    constructor(radiusOrAttribs, previousSpec) {
        super(typeof radiusOrAttribs === "number"
            ? { radius: radiusOrAttribs }
            : radiusOrAttribs, previousSpec);
    }
    /**
     * Processes the surface parts and chain to apply the rounded corners style.
     *
     * @param parts - The surface parts to which the styles will be added.
     * @param chain - The chain of surface elements.
     * @returns A tuple containing the updated surface parts and chain.
     */
    process(parts, chain) {
        return [parts.addContainerStyles(this._getStyle()), chain];
    }
    /**
     * Generates the static style for the rounded corners.
     *
     * @returns A StaticStyle instance with the border radius.
     */
    // FIXME: Use the other attributes
    _getStyle() {
        return new StaticStyle({ borderRadius: this.radius });
    }
}
__decorate([
    memoized
], RoundedCorners.prototype, "_getStyle", null);

/**
 * Generates a StaticStyle for a shadow based on elevation and shade.
 *
 * @param elevation - The elevation level of the shadow.
 * @param tone - The color shade of the shadow.
 * @returns A StaticStyle instance with the computed box shadow.
 */
const getShadowStyle = (elevation, tone) => {
    const blurRadius = elevation === 1 ? 3 : elevation * 2;
    const ambientShadow = {
        x: 0,
        y: elevation,
        blurRadius,
        color: tone.alpha((elevation + 10 + elevation / 9.38) / 100),
    };
    const directShadow = {
        x: 0,
        y: elevation < 10
            ? elevation % 2 === 0
                ? elevation - (elevation / 2 - 1)
                : elevation - (elevation - 1) / 2
            : elevation - 4,
        blurRadius,
        color: tone.alpha((24 - Math.round(elevation / 10)) / 100),
    };
    return new StaticStyle({ boxShadow: [ambientShadow, directShadow] });
};
/**
 * A class representing a DropShadow element for a Surface component.
 */
class DropShadow extends Spec(["elevation", "shade"]) {
    type = "shadow";
    /**
     * Processes the SurfaceParts and element chain to add shadow styles.
     *
     * @param parts - The current SurfaceParts.
     * @param chain - The chain of Surface elements.
     * @returns A tuple containing the updated SurfaceParts and element chain.
     */
    process(parts, chain) {
        const { elevation = 0, shade = black } = this;
        return [
            elevation !== 0
                ? parts.addContainerStyles(getShadowStyle(elevation, shade))
                : parts,
            chain,
        ];
    }
}

/**
 * A class representing a Rectangle shape for a Surface component.
 */
class Rectangle extends Spec(["padding"]) {
    type = "shape";
    /**
     * Constructs a new Rectangle instance.
     *
     * @param attribs - The attributes for the Rectangle.
     * @param previousSpec - The previous Rectangle specification, if any.
     */
    constructor(attribs = {}, previousSpec) {
        super(attribs, previousSpec);
    }
    /**
     * Processes the SurfaceParts and element chain to add container styles.
     *
     * @param parts - The current SurfaceParts.
     * @param chain - The chain of Surface elements.
     * @returns A tuple containing the updated SurfaceParts and element chain.
     */
    process(parts, chain) {
        return [parts.addContainerStyles(this._getStyle()), chain];
    }
    /**
     * Gets the style properties for the Rectangle based on its attributes.
     *
     * @returns A StaticStyle instance with the computed styles.
     */
    _getStyle() {
        const { padding = 0 } = this;
        const styles = {};
        if (typeof padding === "number") {
            styles.padding = `${padding}px`;
        }
        else {
            for (const dir of ["X", "Y", "Left", "Right", "Top", "Bottom"]) {
                const lowerDir = dir.toLowerCase();
                if (lowerDir in padding)
                    styles[`padding${dir}`] = `${padding[lowerDir] ?? 0}px`;
            }
        }
        return new StaticStyle(styles);
    }
}
__decorate([
    memoized
], Rectangle.prototype, "_getStyle", null);

/**
 * A class representing parts of a surface, including prepended and appended
 * components, and container styles. This class provides methods to manipulate
 * these parts immutably.
 */
class SurfaceParts {
    #prependedComponents = [];
    #appendedComponents = [];
    #containerStyles;
    /**
     * Gets the list of prepended components.
     */
    get prependedComponents() {
        return this.#prependedComponents;
    }
    /**
     * Prepends a component to the list of prepended components.
     *
     * @param component - The component to prepend.
     * @returns A new instance of SurfaceParts with the updated list of prepended
     *   components.
     */
    prependComponent(component) {
        const next = this.#clone();
        next.#prependedComponents = [component, ...this.#prependedComponents];
        return next;
    }
    /**
     * Gets the list of appended components.
     */
    get appendedComponents() {
        return this.#appendedComponents;
    }
    /**
     * Appends a component to the list of appended components.
     *
     * @param component - The component to append.
     * @returns A new instance of SurfaceParts with the updated list of appended
     *   components.
     */
    appendComponent(component) {
        const next = this.#clone();
        next.#appendedComponents = [...this.#appendedComponents, component];
        return next;
    }
    /**
     * Gets the container styles.
     */
    get containerStyles() {
        return this.#containerStyles;
    }
    /**
     * Adds styles to the container.
     *
     * @param styles - The styles to add.
     * @returns A new instance of SurfaceParts with the updated container styles.
     */
    addContainerStyles(styles) {
        const next = this.#clone();
        next.#containerStyles = ThemedStyle.concat(this.#containerStyles, styles);
        return next;
    }
    /**
     * Creates a clone of the current instance.
     *
     * @returns A new instance of SurfaceParts with the same properties.
     */
    #clone() {
        const next = new SurfaceParts();
        next.#prependedComponents = this.#prependedComponents;
        next.#appendedComponents = this.#appendedComponents;
        next.#containerStyles = this.#containerStyles;
        return next;
    }
}

/**
 * Names of the Surface elements.
 */
const ELEMENT_NAMES = freeze([
    "shape",
    "corners",
    "border",
    "background",
    "shadow",
]);
/**
 * Names of the Surface attributes.
 */
const ATTRIBUTE_NAMES = freeze([...ELEMENT_NAMES, "foreground"]);
/**
 * A class representing the specification for a Surface component. It includes
 * attributes, state properties, and methods to convert the spec to a React
 * component.
 */
class SurfaceSpec extends Spec(ATTRIBUTE_NAMES) {
    /**
     * Converts the SurfaceSpec to a React component.
     *
     * @example
     *
     * ```tsx
     * const MySurface = surfaceSpec.toComponent();
     * <MySurface state="hovered">Content</MySurface>;
     * ```
     *
     * @returns The React component.
     */
    toComponent() {
        const variantPropKeys = Object.keys(this.variants);
        const hasStateVariant = this.hasVariant("state");
        const SurfaceComponent = TFC.forwardRef(({ children, as, sx, onHoverStart, onHoverEnd, onPress, onPressIn, onPressOut, state, ...props }, ref) => {
            const [internalState, setInternalState] = useState(state ?? "default");
            /**
             * Callback for hover start event.
             */
            const onHoverStartCallback = useCallback((e) => {
                if (state !== "disabled") {
                    if (hasStateVariant)
                        setInternalState("hovered");
                    onHoverStart?.(e);
                }
            }, [state, onHoverStart, setInternalState]);
            /**
             * Callback for hover end event.
             */
            const onHoverEndCallback = useCallback((e) => {
                if (state !== "disabled") {
                    if (hasStateVariant)
                        setInternalState("default");
                    onHoverEnd?.(e);
                }
            }, [state, onHoverEnd, setInternalState]);
            /**
             * Callback for press in event.
             */
            const onPressInCallback = useCallback((e) => {
                if (state !== "disabled") {
                    if (hasStateVariant)
                        setInternalState("pressed");
                    onPressIn?.(e);
                }
            }, [state, onPressIn, setInternalState]);
            /**
             * Callback for press out event.
             */
            const onPressOutCallback = useCallback((e) => {
                if (state !== "disabled") {
                    if (hasStateVariant)
                        setInternalState("hovered");
                    onPressOut?.(e);
                }
            }, [state, onPressOut, setInternalState]);
            const propsWithDefaults = {
                ...props,
                state: state ?? internalState,
            };
            const variantProps = pick(propsWithDefaults, variantPropKeys);
            const { parts, foreground } = this._getSurfaceParts(variantProps);
            return (jsxs(View, { sx: ThemedStyle.concat(parts.containerStyles, sx), as: as, ref: ref, onHoverStart: onHoverStartCallback, onHoverEnd: onHoverEndCallback, onPress: onPress, onPressIn: onPressInCallback, onPressOut: onPressOutCallback, children: [parts.prependedComponents, jsx(ColorProvider, { foreground: foreground, children: children }), parts.appendedComponents] }));
        });
        return SurfaceComponent;
    }
    /**
     * Gets the surface parts and foreground color based on the active variants.
     *
     * @param activeVariants - The active variants.
     * @returns An object containing the surface parts and foreground color.
     */
    _getSurfaceParts(activeVariants) {
        const { foreground, ...attributes } = this.using(activeVariants).attributes;
        let parts = new SurfaceParts();
        let chain = ELEMENT_NAMES.map((name) => attributes[name]).filter(isDefined.test);
        while (chain.length > 0) {
            const element = chain.shift();
            if (element != null)
                [parts, chain] = element.process(parts, chain);
        }
        return { parts, foreground };
    }
}
__decorate([
    memoized
], SurfaceSpec.prototype, "toComponent", null);
__decorate([
    memoized(byVariant)
], SurfaceSpec.prototype, "_getSurfaceParts", null);

/**
 * A hook that retrieves a Surface component from the theme based on the
 * provided names.
 *
 * This hook looks up the surface definitions in the theme using the provided
 * names. It returns the first matching Surface component it finds.
 *
 * @example
 *
 * ```tsx
 * const MySurface = useSurface(['button', 'pressable']);
 * <MySurface>Content</MySurface>
 * ```
 *
 * @param names - An array of names to look up in the theme.
 * @returns The Surface component.
 * @throws If no matching surface definition is found in the theme.
 */
function useSurface(names) {
    const { surfaces } = useTheme();
    const Component = coalesce(...names.map((name) => surfaces[name]))?.toComponent();
    isDefined.assert(Component, () => `Can't find a surface definition for this component in the theme. Looked up keys: ${names
        .map((n) => n.toString())
        .join(", ")}.`);
    return Component;
}

var FontStyle;
(function (FontStyle) {
    FontStyle["Normal"] = "normal";
    FontStyle["Italic"] = "italic";
    FontStyle["Oblique"] = "oblique";
})(FontStyle || (FontStyle = {}));

class FontWeight {
    static Thin = new FontWeight("Thin", 100);
    static ExtraLight = new FontWeight("ExtraLight", 200);
    static Light = new FontWeight("Light", 300);
    static Regular = new FontWeight("Regular", 400);
    static Medium = new FontWeight("Medium", 500);
    static SemiBold = new FontWeight("SemiBold", 600);
    static Bold = new FontWeight("Bold", 700);
    static ExtraBold = new FontWeight("ExtraBold", 800);
    static Black = new FontWeight("Black", 900);
    static for(weight) {
        if (weight > 0 && weight < 10)
            return ORDERED_WEIGHTS[Math.round(weight) - 1];
        if (weight >= 100 && weight <= 900)
            return ORDERED_WEIGHTS[Math.round(weight / 100) - 1];
        return FontWeight.Regular;
    }
    value;
    name;
    #index;
    constructor(name, value) {
        this.name = name;
        this.value = value;
    }
    get index() {
        this.#index ??= ORDERED_WEIGHTS.indexOf(this);
        return this.#index;
    }
    get bolder() {
        return this.index === ORDERED_WEIGHTS.length - 1
            ? void 0
            : ORDERED_WEIGHTS[this.index + 1];
    }
    get lighter() {
        return this.index === 0 ? void 0 : ORDERED_WEIGHTS[this.index + 1];
    }
    get fallbacks() {
        return ORDERED_WEIGHTS.slice(this.index);
    }
    isLighterThan(other) {
        return this.value < other.value;
    }
    isBolderThan(other) {
        return this.value > other.value;
    }
    toString() {
        return this.name;
    }
}
const ORDERED_WEIGHTS = [
    FontWeight.Thin,
    FontWeight.ExtraLight,
    FontWeight.Light,
    FontWeight.Regular,
    FontWeight.Medium,
    FontWeight.SemiBold,
    FontWeight.Bold,
    FontWeight.ExtraBold,
    FontWeight.Black,
];

const fontDesignationMap = new Map();
class FontDesignation {
    static fromFvd(fvd) {
        const style = {
            i: FontStyle.Italic,
            n: FontStyle.Normal,
            o: FontStyle.Oblique,
        }[fvd.charAt(0)] ?? FontStyle.Normal;
        const weight = FontWeight.for(Number.parseInt(fvd.charAt(1)));
        return this.for(weight, style);
    }
    static for(weight, style) {
        const key = toKey(weight, style);
        if (!fontDesignationMap.has(key)) {
            fontDesignationMap.set(key, new FontDesignation(weight, style));
        }
        return fontDesignationMap.get(key);
    }
    weight;
    style;
    constructor(weight, style) {
        this.weight = weight;
        this.style = style;
    }
    get fallbacks() {
        return this.weight.fallbacks.flatMap((weight) => fontStyleAlternatives(this.style).map((style) => FontDesignation.for(weight, style)));
    }
    get fvd() {
        const weight = this.weight.value / 100;
        const style = {
            [FontStyle.Normal]: "n",
            [FontStyle.Italic]: "i",
            [FontStyle.Oblique]: "o",
        }[this.style];
        return `${style}${weight}`;
    }
    get googleFontsDescription() {
        const weight = this.weight.value;
        const style = {
            [FontStyle.Normal]: "",
            [FontStyle.Italic]: "i",
            [FontStyle.Oblique]: "o",
        }[this.style];
        return `${weight}${style}`;
    }
    toString() {
        return toKey(this.weight, this.style);
    }
    toHumanString() {
        return titleize(this.toString());
    }
}
const toKey = (weight, style) => weight.name + (style === FontStyle.Normal ? "" : titleize(style));
const fontStyleAlternatives = (style) => {
    switch (style) {
        case FontStyle.Normal: {
            return [FontStyle.Normal];
        }
        case FontStyle.Italic: {
            return [FontStyle.Italic, FontStyle.Oblique];
        }
        case FontStyle.Oblique: {
            return [FontStyle.Oblique, FontStyle.Italic];
        }
    }
};

class Font {
    family;
    weight;
    style;
    fontLoader;
    constructor(family, weight, style, fontLoader) {
        this.family = family;
        this.weight = weight;
        this.style = style;
        this.fontLoader = fontLoader;
    }
    withSize(size) {
        return new FontSpec({ font: this, size });
    }
    with(attributeNameOrValues, value) {
        if (isString(attributeNameOrValues)) {
            isDefined.assert(value);
            return this.withSize(value);
        }
        else {
            return new FontSpec({ ...attributeNameOrValues, font: this });
        }
    }
    get designation() {
        return FontDesignation.for(this.weight, this.style);
    }
    get bolder() {
        const bolderWeight = this.weight.bolder;
        return bolderWeight != null
            ? (this.family.getClosestFont(bolderWeight, this.style) ?? this)
            : this;
    }
    get lighter() {
        const lighterWeight = this.weight.lighter;
        return lighterWeight != null
            ? (this.family.getClosestFont(lighterWeight, this.style) ?? this)
            : this;
    }
    get italic() {
        // TODO: Should fallback to a generic font?
        return this.family.getClosestFont(this.weight, FontStyle.Italic) ?? this;
    }
    get normal() {
        // TODO: Should fallback to a generic font?
        return this.family.getClosestFont(this.weight, FontStyle.Normal) ?? this;
    }
    async load() {
        return this.fontLoader.load(this);
    }
    isLoaded() {
        return this.fontLoader.isLoaded(this);
    }
    toString() {
        return `${this.family.name} ${this.designation.toHumanString()}`;
    }
}
class FontSpec extends Spec([
    "size",
    "lineHeight",
    "font",
    "letterSpacing",
    "textTransform",
    "textAlign",
]) {
    get allFonts() {
        const variantDefiner = this.variantDefiner;
        const combinations = variantDefiner.getCombinations();
        const fontSet = combinations.reduce((set, activeVariants) => set.add(this.using(activeVariants).font), new Set());
        return [...fontSet];
    }
    toStaticStyle(_theme) {
        const { size, letterSpacing, font, lineHeight, textTransform, textAlign } = this;
        return new StaticStyle(compactObject({
            fontSize: Math.round(size),
            letterSpacing,
            fontFamily: font.family.toString(),
            fontWeight: font.weight.value,
            fontStyle: font.style,
            lineHeight,
            textTransform,
            textAlign,
        }));
    }
}
__decorate([
    memoized(byTheme)
], FontSpec.prototype, "toStaticStyle", null);

var FontType;
(function (FontType) {
    FontType["Cursive"] = "cursive";
    FontType["Fantasy"] = "fantasy";
    FontType["Monospace"] = "monospace";
    FontType["SansSerif"] = "sans-serif";
    FontType["Serif"] = "serif";
})(FontType || (FontType = {}));
class FontFamily {
    name;
    fallback;
    fontType;
    #fontMap = new Map();
    constructor(name, fontType, fallback = []) {
        this.name = name;
        this.fontType = fontType;
        this.fallback = fallback;
    }
    withFont(weight, style) {
        const designation = FontDesignation.for(weight, style);
        const font = new Font(this, weight, style, this.fontLoader);
        this.#fontMap.set(designation, font);
        Object.defineProperty(this, designation.toString(), { get: () => font });
        return this;
    }
    /**
     * List of the fonts belonging to this family.
     */
    get fonts() {
        return [...new Set(this.#fontMap.values())];
    }
    get weights() {
        return [...new Set(this.fonts.map((font) => font.weight))];
    }
    get styles() {
        return [...new Set(this.fonts.map((font) => font.style))];
    }
    getFont(weightOrDesignation, style) {
        if (weightOrDesignation instanceof FontDesignation) {
            return this.#fontMap.get(weightOrDesignation);
        }
        else {
            isDefined.assert(style);
            return this.#fontMap.get(FontDesignation.for(weightOrDesignation, style));
        }
    }
    getClosestFont(weight, style) {
        const fallbacks = FontDesignation.for(weight, style).fallbacks;
        for (const fb of fallbacks) {
            if (this.#fontMap.has(fb))
                return this.#fontMap.get(fb);
        }
    }
    toString() {
        return [this.name, ...this.fallback, this.fontType]
            .map((t) => {
            const name = t instanceof FontFamily ? t.name : t;
            return name.includes(" ") ? `"${name}"` : name;
        })
            .join(", ");
    }
}

var TextTransform;
(function (TextTransform) {
    TextTransform["None"] = "none";
    TextTransform["Uppercase"] = "uppercase";
    TextTransform["Lowercase"] = "lowercase";
    TextTransform["Capitalize"] = "capitalize";
})(TextTransform || (TextTransform = {}));

function useTypography(specNames) {
    const { typography } = useTheme();
    const validSpecName = asArray(specNames).find((name) => typography[name] != null);
    return validSpecName != null ? typography[validSpecName] : undefined;
}

export { Color, ColorConsumer, ColorPicker, ColorProvider, ColorScale, ComputedTheme, ContrastLevel, DropShadow, FixedColorScale, Font, FontDesignation, FontFamily, FontSpec, FontStyle, FontType, FontWeight, Image, Input, Rectangle, RoundedCorners, Shade, SolidBorder, SolidFill, StaticStyle, ThemedStyle as Style, SurfaceParts, SurfaceSpec, TFC, Text, TextTransform, Theme, ThemeConsumer, ThemeProvider, ThemedStyle, UniformLightnessColorScale, Upload, View, aliceBlue, antiqueWhite, aqua, aquamarine, azure, beige, bisque, black, blanchedAlmond, blue, blueViolet, brown, burlywood, cadetBlue, chartreuse, chocolate, coral, cornflowerBlue, cornsilk, createThemeSxProcessor, crimson, cyan, darkBlue, darkCyan, darkGoldenrod, darkGray, darkGreen, darkKhaki, darkMagenta, darkOliveGreen, darkOrange, darkOrchid, darkRed, darkSalmon, darkSeaGreen, darkSlateBlue, darkSlateGray, darkTurquoise, darkViolet, deepPink, deepSkyBlue, dimGray, dodgerBlue, easeInCubic, easeInOutCubic, easeInOutQuad, easeInOutQuart, easeInOutQuint, easeInQuad, easeInQuart, easeInQuint, easeOutCubic, easeOutQuad, easeOutQuart, easeOutQuint, firebrick, floralWhite, forestGreen, fuchsia, gainsboro, getWCAGContrast, ghostWhite, gold, goldenrod, gray, green, greenYellow, hex, honeydew, hotPink, hsl, hsla, indianRed, indigo, isConvertibleToStyle, isHexColorString, ivory, khaki, lavender, lavenderBlush, lawnGreen, lemonChiffon, lightBlue, lightCoral, lightCyan, lightGoldenrodYellow, lightGray, lightGreen, lightPink, lightSalmon, lightSeaGreen, lightSkyBlue, lightSlateGray, lightSteelBlue, lightYellow, lime, limeGreen, linear, linen, magenta, maroon, mediumAquamarine, mediumBlue, mediumOrchid, mediumPurple, mediumSeaGreen, mediumSlateBlue, mediumSpringGreen, mediumTurquoise, mediumVioletRed, midnightBlue, mintCream, mistyRose, moccasin, navajoWhite, navy, oldLace, olive, oliveDrab, orange, orangeRed, orchid, paleGoldenrod, paleGreen, paleTurquoise, paleVioletRed, papayaWhip, parseHexColorString, peachPuff, peru, pink, plum, powderBlue, purple, red, resolveThemedStyleProperties, rgb, rgba, rosyBrown, royalBlue, saddleBrown, salmon, sandyBrown, seaGreen, seashell, sienna, silver, skyBlue, slateBlue, slateGray, snow, springGreen, steelBlue, tan, tanh, teal, thistle, toHexColorString, tomato, transparent, turquoise, u, units, useCurrentBackground, useCurrentForeground, useCurrentPalette, usePalette, useStaticStyle, useSurface, useTheme, useThemedStyle, useTypography, violet, wheat, white, whiteSmoke, yellow, yellowGreen };
